@@ -21,6 +21,7 @@
 #include <strings.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <malloc.h>
 #include <sys/types.h>
 #include "error.h"
@@ -38,9 +39,8 @@
 
 extern int verbose;
 
-static const char * const deb_checksum_headers[cs_COUNT] = {"MD5sum", "SHA1",
-	// "SHA256",
-	"Size"};
+static const char * const deb_checksum_headers[cs_COUNT] = {
+	"MD5sum", "SHA1", "SHA256", "Size"};
 
 /* get checksums out of a "Packages"-chunk. */
 static retvalue binaries_parse_checksums(const char *chunk, /*@out@*/struct checksums **checksums_p) {
@@ -57,11 +57,13 @@ static retvalue binaries_parse_checksums(const char *chunk, /*@out@*/struct chec
 		RET_UPDATE(result, r);
 	}
 	if( checksums[cs_md5sum] == NULL ) {
-		fprintf(stderr,"Missing 'MD5sum'-line in binary control chunk:\n '%s'\n",chunk);
+		fprintf(stderr, "Missing 'MD5sum' line in binary control chunk:\n '%s'\n",
+				chunk);
 		RET_UPDATE(result, RET_ERROR_MISSING);
 	}
 	if( checksums[cs_length] == NULL ) {
-		fprintf(stderr,"Missing 'Size'-line in binary control chunk:\n '%s'\n",chunk);
+		fprintf(stderr, "Missing 'Size' line in binary control chunk:\n '%s'\n",
+				chunk);
 		RET_UPDATE(result, RET_ERROR_MISSING);
 	}
 	if( RET_WAS_ERROR(result) ) {
@@ -73,7 +75,7 @@ static retvalue binaries_parse_checksums(const char *chunk, /*@out@*/struct chec
 }
 
 /* get somefields out of a "Packages.gz"-chunk. returns RET_OK on success, RET_NOTHING if incomplete, error otherwise */
-static retvalue binaries_parse_chunk(const char *chunk,const char *packagename,const char *packagetype,const char *version,/*@out@*/char **sourcename,/*@out@*/char **basename) {
+static retvalue binaries_parse_chunk(const char *chunk, const char *packagename, const char *packagetype, const char *version, /*@out@*/char **sourcename, /*@out@*/char **basename, /*@out@*/enum filetype *ft_p) {
 	retvalue r;
 	char *parch;
 	char *mysourcename,*mybasename;
@@ -97,6 +99,10 @@ static retvalue binaries_parse_chunk(const char *chunk,const char *packagename,c
 		free(mysourcename);
 		return r;
 	}
+	if( strcmp(parch, "all") == 0 )
+		*ft_p = ft_ALL_BINARY;
+	else
+		*ft_p = ft_ARCH_BINARY;
 	r = properpackagename(packagename);
 	if( !RET_WAS_ERROR(r) )
 		r = properversion(version);
@@ -127,7 +133,8 @@ retvalue binaries_getfilekeys(const char *chunk, struct strlist *files) {
 	r = chunk_getvalue(chunk,"Filename",&filename);
 	if( !RET_IS_OK(r) ) {
 		if( r == RET_NOTHING ) {
-			fprintf(stderr,"Does not look like binary control: '%s'\n",chunk);
+			fprintf(stderr, "Data does not look like binary control: '%s'\n",
+					chunk);
 			r = RET_ERROR;
 		}
 		return r;
@@ -167,41 +174,43 @@ static inline retvalue calcnewcontrol(const char *chunk,const char *sourcename,c
 	return RET_OK;
 }
 
-retvalue binaries_getname(UNUSED(struct target *t),const char *control,char **packagename){
+retvalue binaries_getname(const char *control, char **packagename){
 	retvalue r;
 
 	r = chunk_getvalue(control,"Package",packagename);
 	if( RET_WAS_ERROR(r) )
 		return r;
 	if( r == RET_NOTHING ) {
-		fprintf(stderr,"Did not find Package name in chunk:'%s'\n",control);
+		fprintf(stderr, "Missing 'Package' field in chunk:'%s'\n", control);
 		return RET_ERROR;
 	}
 	return r;
 }
-retvalue binaries_getversion(UNUSED(struct target *t),const char *control,char **version) {
+retvalue binaries_getversion(const char *control, char **version) {
 	retvalue r;
 
 	r = chunk_getvalue(control,"Version",version);
 	if( RET_WAS_ERROR(r) )
 		return r;
 	if( r == RET_NOTHING ) {
-		fprintf(stderr,"Did not find Version in chunk:'%s'\n",control);
+		fprintf(stderr, "Missing 'Version' field in chunk:'%s'\n", control);
 		return RET_ERROR;
 	}
 	return r;
 }
 
-retvalue binaries_getinstalldata(struct target *t, const char *packagename, const char *version, const char *chunk, char **control, struct strlist *filekeys, struct checksumsarray *origfiles) {
+retvalue binaries_getinstalldata(const struct target *t, const char *packagename, const char *version, const char *chunk, char **control, struct strlist *filekeys, struct checksumsarray *origfiles, /*@null@*//*@out@*/enum filetype *type_p) {
 	char *sourcename IFSTUPIDCC(=NULL) ,*basename IFSTUPIDCC(=NULL);
 	struct checksumsarray origfilekeys;
 	retvalue r;
+	enum filetype ft IFSTUPIDCC(='?');
 
-	r = binaries_parse_chunk(chunk,packagename,t->packagetype,version,&sourcename,&basename);
+	r = binaries_parse_chunk(chunk, packagename, t->packagetype,
+			version, &sourcename, &basename, &ft);
 	if( RET_WAS_ERROR(r) ) {
 		return r;
 	} else if( r == RET_NOTHING ) {
-		fprintf(stderr,"Does not look like a binary package: '%s'!\n",chunk);
+		fprintf(stderr, "Does not look like a binary package: '%s'!\n", chunk);
 		return RET_ERROR;
 	}
 	r = binaries_getchecksums(chunk, &origfilekeys);
@@ -210,13 +219,16 @@ retvalue binaries_getinstalldata(struct target *t, const char *packagename, cons
 		return r;
 	}
 
-	r = calcnewcontrol(chunk,sourcename,basename,t->component,filekeys,control);
+	r = calcnewcontrol(chunk, sourcename, basename,
+			t->component, filekeys, control);
 	if( RET_WAS_ERROR(r) ) {
 		checksumsarray_done(&origfilekeys);
 	} else {
 		assert( r != RET_NOTHING );
 		checksumsarray_move(origfiles, &origfilekeys);
 	}
+	if( type_p != NULL )
+		*type_p = ft;
 	free(sourcename);free(basename);
 	return r;
 }
@@ -245,12 +257,10 @@ retvalue binaries_getchecksums(const char *chunk, struct checksumsarray *filekey
 	return RET_OK;
 }
 
-char *binaries_getupstreamindex(UNUSED(struct target *target),const char *suite_from,
-		const char *component_from,const char *architecture) {
+char *binaries_getupstreamindex(const char *suite_from, const char *component_from, const char *architecture) {
 	return mprintf("dists/%s/%s/binary-%s/Packages.gz",suite_from,component_from,architecture);
 }
-char *ubinaries_getupstreamindex(UNUSED(struct target *target),const char *suite_from,
-		const char *component_from,const char *architecture) {
+char *ubinaries_getupstreamindex(const char *suite_from, const char *component_from, const char *architecture) {
 	return mprintf("dists/%s/%s/debian-installer/binary-%s/Packages.gz",suite_from,component_from,architecture);
 }
 
@@ -333,14 +343,16 @@ retvalue binaries_retrack(const char *packagename, const char *chunk, trackingdb
 		}
 		if( r == RET_NOTHING ) {
 			free(fsourcename);
-			fprintf(stderr,"Did not find Version in chunk:'%s'\n",chunk);
+			fprintf(stderr, "Missing 'Version' field in chunk:'%s'\n",
+					chunk);
 			return RET_ERROR;
 		}
 	}
 
 	r = chunk_getvalue(chunk,"Architecture",&arch);
 	if( r == RET_NOTHING ) {
-		fprintf(stderr,"Did not find Architecture in chunk:'%s'\n",chunk);
+		fprintf(stderr, "No Architecture field in chunk:'%s'\n",
+				chunk);
 		r = RET_ERROR;
 	}
 	if( RET_WAS_ERROR(r) ) {
@@ -358,7 +370,8 @@ retvalue binaries_retrack(const char *packagename, const char *chunk, trackingdb
 	r = chunk_getvalue(chunk,"Filename",&filekey);
 	if( !RET_IS_OK(r) ) {
 		if( r == RET_NOTHING ) {
-			fprintf(stderr,"Did not find a Filename in chunk: '%s'\n",chunk);
+			fprintf(stderr, "No Filename field in chunk: '%s'\n",
+					chunk);
 			r = RET_ERROR;
 		}
 		free(sourceversion);
@@ -382,7 +395,7 @@ retvalue binaries_retrack(const char *packagename, const char *chunk, trackingdb
 	return tracking_save(tracks, pkg);
 }
 
-retvalue binaries_getsourceandversion(UNUSED(struct target *t),const char *chunk,const char *packagename,char **source,char **version) {
+retvalue binaries_getsourceandversion(const char *chunk, const char *packagename, char **source, char **version) {
 	retvalue r;
 	char *sourcename,*sourceversion;
 
@@ -407,7 +420,7 @@ retvalue binaries_getsourceandversion(UNUSED(struct target *t),const char *chunk
 		}
 		if( r == RET_NOTHING ) {
 			free(sourcename);
-			fprintf(stderr,"Did not find Version in chunk:'%s'\n",chunk);
+			fprintf(stderr, "No Version field in chunk:'%s'\n", chunk);
 			return RET_ERROR;
 		}
 	}
@@ -421,7 +434,8 @@ static inline retvalue getvalue(const char *filename,const char *chunk,const cha
 
 	r = chunk_getvalue(chunk,field,value);
 	if( r == RET_NOTHING ) {
-		fprintf(stderr,"Cannot find %s-header in control file of %s!\n",field,filename);
+		fprintf(stderr, "No %s field in %s's control file!\n",
+				field, filename);
 		r = RET_ERROR;
 	}
 	return r;
@@ -432,7 +446,8 @@ static inline retvalue checkvalue(const char *filename,const char *chunk,const c
 
 	r = chunk_checkfield(chunk,field);
 	if( r == RET_NOTHING ) {
-		fprintf(stderr,"Cannot find %s-header in control file of %s!\n",field,filename);
+		fprintf(stderr, "No %s field in %s's control file!\n",
+				field, filename);
 		r = RET_ERROR;
 	}
 	return r;
@@ -467,7 +482,7 @@ retvalue binaries_readdeb(struct deb_headers *deb, const char *filename, bool ne
 
 	r = chunk_getname(deb->control, "Package", &deb->name, false);
 	if( r == RET_NOTHING ) {
-		fprintf(stderr,"Missing 'Package' field in %s!\n",filename);
+		fprintf(stderr, "Missing 'Package' field in %s!\n", filename);
 		r = RET_ERROR;
 	}
 	if( RET_WAS_ERROR(r) )
