@@ -20,6 +20,7 @@
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 #include <malloc.h>
 #include <sys/types.h>
 #include "error.h"
@@ -27,7 +28,7 @@
 #include "checksums.h"
 #include "changes.h"
 
-retvalue changes_parsefileline(const char *fileline, /*@out@*/filetype *result_type, /*@out@*/char **result_basename, /*@out@*/struct checksums **result_checksums, /*@out@*/char **result_section, /*@out@*/char **result_priority, /*@out@*/char **result_architecture, /*@out@*/char **result_name) {
+retvalue changes_parsefileline(const char *fileline, /*@out@*/filetype *result_type, /*@out@*/char **result_basename, /*@out@*/struct hash_data *hash_p, /*@out@*/struct hash_data *size_p, /*@out@*/char **result_section, /*@out@*/char **result_priority, /*@out@*/char **result_architecture, /*@out@*/char **result_name) {
 
 	const char *p,*md5start,*md5end;
 	const char *sizestart,*sizeend;
@@ -38,7 +39,6 @@ retvalue changes_parsefileline(const char *fileline, /*@out@*/filetype *result_t
 	const char *versionstart,*typestart;
 	filetype type;
 	char *section, *priority, *basename, *architecture, *name;
-	retvalue r;
 
 	p = fileline;
 	while( *p !='\0' && xisspace(*p) )
@@ -57,6 +57,8 @@ retvalue changes_parsefileline(const char *fileline, /*@out@*/filetype *result_t
 	md5end = p;
 	while( *p !='\0' && xisspace(*p) )
 		p++;
+	while( *p == '0' && p[1] >= '0' && p[1] <= '9' )
+		p++;
 	sizestart = p;
 	while( *p >= '0' && *p <= '9' )
 		p++;
@@ -95,7 +97,8 @@ retvalue changes_parsefileline(const char *fileline, /*@out@*/filetype *result_t
 	}
 	if( *md5start == '\0' || *sizestart == '\0' || *sectionstart == '\0'
 			|| *priostart == '\0' || *filestart == '\0' ) {
-		fprintf(stderr,"Not five arguments in '%s'!\n",fileline);
+		fprintf(stderr, "Wrong number of arguments in '%s' (5 expected)!\n",
+				fileline);
 		return RET_ERROR;
 	}
 
@@ -104,9 +107,11 @@ retvalue changes_parsefileline(const char *fileline, /*@out@*/filetype *result_t
 		p++;
 	if( *p != '_' ) {
 		if( *p == '\0' )
-			fprintf(stderr,"No underscore in filename in '%s'!\n",fileline);
+			fprintf(stderr, "No underscore found in file name in '%s'!\n",
+					fileline);
 		else
-			fprintf(stderr,"Unexpected character '%c' in filename in '%s'!\n",*p,fileline);
+			fprintf(stderr, "Unexpected character '%c' in file name in '%s'!\n",
+					*p, fileline);
 		return RET_ERROR;
 	}
 	nameend = p;
@@ -117,7 +122,8 @@ retvalue changes_parsefileline(const char *fileline, /*@out@*/filetype *result_t
 	// this check gets the broken things.
 	names_overversion(&p, true);
 	if( *p != '\0' && *p != '_' ) {
-		fprintf(stderr,"Unexpected character '%c' in filename within '%s'!\n",*p,fileline);
+		fprintf(stderr, "Unexpected character '%c' in file name within '%s'!\n",
+				*p, fileline);
 		return RET_ERROR;
 	}
 	if( *p == '_' ) {
@@ -128,7 +134,8 @@ retvalue changes_parsefileline(const char *fileline, /*@out@*/filetype *result_t
 		while( *p !='\0' && *p != '.' )
 			p++;
 		if( *p != '.' ) {
-			fprintf(stderr,"Expect something of the form name_version_arch.[u]deb but got '%s'!\n",filestart);
+			fprintf(stderr,
+"'%s' is not of the expected form name_version_arch.[u]deb!\n", filestart);
 			return RET_ERROR;
 		}
 		archend = p;
@@ -141,11 +148,12 @@ retvalue changes_parsefileline(const char *fileline, /*@out@*/filetype *result_t
 		else if( p-typestart == 4 && strncmp(typestart,"udeb",4) == 0 )
 			type = fe_UDEB;
 		else {
-			fprintf(stderr,"'%s' looks neighter like .deb nor like .udeb!\n",filestart);
+			fprintf(stderr, "'%s' is not .deb or .udeb!\n", filestart);
 			return RET_ERROR;
 		}
 		if( strncmp(archstart,"source",6) == 0 ) {
-			fprintf(stderr,"How can a .[u]deb be architecture 'source'?('%s')\n",filestart);
+			fprintf(stderr,
+"Architecture 'source' not allowed for .[u]debs ('%s')!\n", filestart);
 			return RET_ERROR;
 		}
 	} else {
@@ -176,7 +184,8 @@ retvalue changes_parsefileline(const char *fileline, /*@out@*/filetype *result_t
 			type = fe_DIFF;
 		else {
 			type = fe_UNKNOWN;
-			fprintf(stderr,"Unknown filetype: '%s', assuming to be source format...\n",fileline);
+			fprintf(stderr,
+"Unknown file type: '%s', assuming source format...\n", fileline);
 		}
 		archstart = "source";
 		archend = archstart + 6;
@@ -192,14 +201,10 @@ retvalue changes_parsefileline(const char *fileline, /*@out@*/filetype *result_t
 		free(basename); free(architecture); free(name);
 		return RET_ERROR_OOM;
 	}
-	r = checksums_set(result_checksums, md5start, md5end - md5start,
-			sizestart, sizeend - sizestart);
-	assert( r != RET_NOTHING );
-	if( RET_WAS_ERROR(r) ) {
-		free(section); free(priority);
-		free(basename); free(architecture); free(name);
-		return r;
-	}
+	hash_p->start = md5start;
+	hash_p->len = md5end - md5start;
+	size_p->start = sizestart;
+	size_p->len = sizeend - sizestart;
 	*result_section = section;
 	*result_priority = priority;
 	*result_basename = basename;
