@@ -1,5 +1,5 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2003,2004,2005,2006,2007 Bernhard R. Link
+ *  Copyright (C) 2003,2004,2005,2006,2007,2008 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
@@ -49,6 +49,7 @@ static retvalue distribution_free(struct distribution *distribution) {
 	if( distribution != NULL ) {
 		free(distribution->codename);
 		free(distribution->suite);
+		free(distribution->fakecomponentprefix);
 		free(distribution->version);
 		free(distribution->origin);
 		free(distribution->notautomatic);
@@ -93,7 +94,7 @@ static retvalue distribution_free(struct distribution *distribution) {
 		return RET_OK;
 }
 
-/* allow premature free'ing of overrides to save some memorty */
+/* allow premature free'ing of overrides to save some memory */
 void distribution_unloadoverrides(struct distribution *distribution) {
 	override_free(distribution->overrides.deb);
 	override_free(distribution->overrides.udeb);
@@ -169,7 +170,6 @@ static retvalue createtargets(struct distribution *distribution) {
 }
 
 struct read_distribution_data {
-	const char *logdir;
 	struct distribution *distributions;
 };
 
@@ -329,14 +329,15 @@ CFexportmodeSETPROC(distribution, dsc)
 CFcheckuniqstrlistSETPROC(distribution, components, checkforcomponent)
 CFcheckuniqstrlistSETPROC(distribution, architectures, checkforarchitecture)
 CFcheckvalueSETPROC(distribution, codename, checkforcodename)
+CFcheckvalueSETPROC(distribution, fakecomponentprefix, checkfordirectoryandidentifier)
 
 CFUSETPROC(distribution, Contents) {
 	CFSETPROCVAR(distribution, d);
 	return contentsoptions_parse(d, iter);
 }
-CFuSETPROC(distribution, logger) {
-	CFSETPROCVARS(distribution, d, mydata);
-	return logger_init(confdir, mydata->logdir, iter, &d->logger);
+CFUSETPROC(distribution, logger) {
+	CFSETPROCVAR(distribution, d);
+	return logger_init(iter, &d->logger);
 }
 CFUSETPROC(distribution, Tracking) {
 	CFSETPROCVAR(distribution, d);
@@ -357,6 +358,7 @@ static const struct configfield distributionconfigfields[] = {
 	CF("Description",	distribution,	description),
 	CF("DscIndices",	distribution,	dsc),
 	CF("DscOverride",	distribution,	dsc_override),
+	CF("FakeComponentPrefix", distribution,	fakecomponentprefix),
 	CF("Label",		distribution,	label),
 	CF("Log",		distribution,	logger),
 	CF("NotAutomatic",	distribution,	notautomatic),
@@ -374,11 +376,10 @@ static const struct configfield distributionconfigfields[] = {
 };
 
 /* read specification of all distributions */
-retvalue distribution_readall(const char *confdir, const char *logdir, struct distribution **distributions) {
+retvalue distribution_readall(struct distribution **distributions) {
 	struct read_distribution_data mydata;
 	retvalue result;
 
-	mydata.logdir = logdir;
 	mydata.distributions = NULL;
 
 	// TODO: readd some way to tell about -b or --confdir here?
@@ -394,21 +395,21 @@ retvalue distribution_readall(const char *confdir, const char *logdir, struct di
 	}
 	*/
 
-	result = configfile_parse(confdir, "distributions",
+	result = configfile_parse("distributions",
 			IGNORABLE(unknownfield),
 			startparsedistribution, finishparsedistribution,
 			distributionconfigfields,
 			ARRAYCOUNT(distributionconfigfields),
 			&mydata);
 	if( result == RET_ERROR_UNKNOWNFIELD )
-		fprintf(stderr, "To ignore unknown fields use --ignore=unknownfield\n");
+		fprintf(stderr, "Use --ignore=unknownfield to ignore unknown fields\n");
 	if( RET_WAS_ERROR(result) ) {
 		distribution_freelist(mydata.distributions);
 		return result;
 	}
 	if( mydata.distributions == NULL ) {
 		fprintf(stderr, "No distribution definitions found in %s/distributions!\n",
-				confdir);
+				global.confdir);
 		distribution_freelist(mydata.distributions);
 		return RET_ERROR_MISSING;
 	}
@@ -416,27 +417,6 @@ retvalue distribution_readall(const char *confdir, const char *logdir, struct di
 	return RET_OK;
 }
 
-/* call <action> for each part of <distribution>. */
-retvalue distribution_foreach_part(struct distribution *distribution,const char *component,const char *architecture,const char *packagetype,distribution_each_action action,void *data) {
-	retvalue result,r;
-	struct target *t;
-
-	result = RET_NOTHING;
-	for( t = distribution->targets ; t != NULL ; t = t->next ) {
-		if( component != NULL && strcmp(component,t->component) != 0 )
-			continue;
-		if( architecture != NULL && strcmp(architecture,t->architecture) != 0 )
-			continue;
-		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
-			continue;
-		r = action(data,t,distribution);
-		RET_UPDATE(result,r);
-		if( RET_WAS_ERROR(r) )
-			return result;
-	}
-	return result;
-}
-
 /* call <action> for each part of <distribution>, within initpackagesdb/closepackagesdb */
 retvalue distribution_foreach_rwopenedpart(struct distribution *distribution,struct database *database,const char *component,const char *architecture,const char *packagetype,distribution_each_action action,void *data) {
 	retvalue result,r;
@@ -444,11 +424,7 @@ retvalue distribution_foreach_rwopenedpart(struct distribution *distribution,str
 
 	result = RET_NOTHING;
 	for( t = distribution->targets ; t != NULL ; t = t->next ) {
-		if( component != NULL && strcmp(component,t->component) != 0 )
-			continue;
-		if( architecture != NULL && strcmp(architecture,t->architecture) != 0 )
-			continue;
-		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
+		if( !target_matches(t, component, architecture, packagetype) )
 			continue;
 		r = target_initpackagesdb(t, database, READWRITE);
 		RET_UPDATE(result, r);
@@ -475,11 +451,7 @@ retvalue distribution_foreach_roopenedpart(struct distribution *distribution,str
 
 	result = RET_NOTHING;
 	for( t = distribution->targets ; t != NULL ; t = t->next ) {
-		if( component != NULL && strcmp(component,t->component) != 0 )
-			continue;
-		if( architecture != NULL && strcmp(architecture,t->architecture) != 0 )
-			continue;
-		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
+		if( !target_matches(t, component, architecture, packagetype) )
 			continue;
 		r = target_initpackagesdb(t, database, READONLY);
 		RET_UPDATE(result, r);
@@ -499,16 +471,12 @@ retvalue distribution_foreach_roopenedpart(struct distribution *distribution,str
 retvalue distribution_foreach_package(struct distribution *distribution, struct database *database, const char *component, const char *architecture, const char *packagetype, each_package_action action, each_target_action target_action, void *data) {
 	retvalue result,r;
 	struct target *t;
-	struct cursor *cursor;
+	struct target_cursor iterator IFSTUPIDCC(=TARGET_CURSOR_ZERO);
 	const char *package, *control;
 
 	result = RET_NOTHING;
 	for( t = distribution->targets ; t != NULL ; t = t->next ) {
-		if( component != NULL && strcmp(component,t->component) != 0 )
-			continue;
-		if( architecture != NULL && strcmp(architecture,t->architecture) != 0 )
-			continue;
-		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
+		if( !target_matches(t, component, architecture, packagetype) )
 			continue;
 		if( target_action != NULL ) {
 			r = target_action(database, distribution, t, data);
@@ -517,27 +485,18 @@ retvalue distribution_foreach_package(struct distribution *distribution, struct
 			if( r == RET_NOTHING )
 				continue;
 		}
-		r = target_initpackagesdb(t, database, READONLY);
+		r = target_openiterator(t, database, READONLY, &iterator);
 		RET_UPDATE(result, r);
 		if( RET_WAS_ERROR(r) )
 			return result;
-		r = table_newglobalcursor(t->packages, &cursor);
-		assert( r != RET_NOTHING );
-		if( RET_WAS_ERROR(r) ) {
-			(void)target_closepackagesdb(t);
-			return r;
-		}
-		while( cursor_nexttemp(t->packages, cursor,
-					&package, &control) ) {
+		while( target_nextpackage(&iterator, &package, &control) ) {
 			r = action(database, distribution, t,
 					package, control, data);
 			RET_UPDATE(result, r);
 			if( RET_WAS_ERROR(r) )
 				break;
 		}
-		r = cursor_close(t->packages, cursor);
-		RET_ENDUPDATE(result, r);
-		r = target_closepackagesdb(t);
+		r = target_closeiterator(&iterator);
 		RET_ENDUPDATE(result, r);
 		if( RET_WAS_ERROR(result) )
 			return result;
@@ -548,8 +507,8 @@ retvalue distribution_foreach_package(struct distribution *distribution, struct
 retvalue distribution_foreach_package_c(struct distribution *distribution, struct database *database, const struct strlist *components, const char *architecture, const char *packagetype, each_package_action action, void *data) {
 	retvalue result,r;
 	struct target *t;
-	struct cursor *cursor;
 	const char *package, *control;
+	struct target_cursor iterator IFSTUPIDCC(=TARGET_CURSOR_ZERO);
 
 	result = RET_NOTHING;
 	for( t = distribution->targets ; t != NULL ; t = t->next ) {
@@ -559,27 +518,18 @@ retvalue distribution_foreach_package_c(struct distribution *distribution, struc
 			continue;
 		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
 			continue;
-		r = target_initpackagesdb(t, database, READONLY);
+		r = target_openiterator(t, database, READONLY, &iterator);
 		RET_UPDATE(result, r);
 		if( RET_WAS_ERROR(r) )
 			return result;
-		r = table_newglobalcursor(t->packages, &cursor);
-		assert( r != RET_NOTHING );
-		if( RET_WAS_ERROR(r) ) {
-			(void)target_closepackagesdb(t);
-			return r;
-		}
-		while( cursor_nexttemp(t->packages, cursor,
-					&package, &control) ) {
+		while( target_nextpackage(&iterator, &package, &control) ) {
 			r = action(database, distribution, t,
 					package, control, data);
 			RET_UPDATE(result, r);
 			if( RET_WAS_ERROR(r) )
 				break;
 		}
-		r = cursor_close(t->packages, cursor);
-		RET_ENDUPDATE(result, r);
-		r = target_closepackagesdb(t);
+		r = target_closeiterator(&iterator);
 		RET_ENDUPDATE(result, r);
 		if( RET_WAS_ERROR(result) )
 			return result;
@@ -616,7 +566,8 @@ struct target *distribution_getpart(const struct distribution *distribution,cons
 /* mark all distributions matching one of the first argc argv */
 retvalue distribution_match(struct distribution *alldistributions, int argc, const char *argv[], bool lookedat) {
 	struct distribution *d;
-	bool *found;
+	bool found[argc], unusable_as_suite[argc];
+	struct distribution *has_suite[argc];
 	int i;
 
 	assert( alldistributions != NULL );
@@ -628,9 +579,9 @@ retvalue distribution_match(struct distribution *alldistributions, int argc, con
 		}
 		return RET_OK;
 	}
-	found = calloc(argc, sizeof(bool));
-	if( found == NULL )
-		return RET_ERROR_OOM;
+	memset(found, 0, sizeof(found));
+	memset(unusable_as_suite, 0, sizeof(unusable_as_suite));
+	memset(has_suite, 0, sizeof(has_suite));
 
 	for( d = alldistributions ; d != NULL ; d = d->next ) {
 		for( i = 0 ; i < argc ; i++ ) {
@@ -640,28 +591,56 @@ retvalue distribution_match(struct distribution *alldistributions, int argc, con
 				d->selected = true;
 				if( lookedat )
 					d->lookedat = lookedat;
+			} else if( d->suite != NULL &&
+					strcmp(argv[i], d->suite) == 0 ) {
+				if( has_suite[i] != NULL )
+					unusable_as_suite[i] = true;
+				has_suite[i] = d;
 			}
 		}
 	}
 	for( i = 0 ; i < argc ; i++ ) {
 		if( !found[i] ) {
+			if( has_suite[i] != NULL && !unusable_as_suite[i] ) {
+				has_suite[i]->selected = true;
+				if( lookedat )
+					has_suite[i]->lookedat = lookedat;
+				continue;
+			}
 			fprintf(stderr, "No distribution definition of '%s' found in distributions'!\n", argv[i]);
-			free(found);
+			if( unusable_as_suite[i] )
+				fprintf(stderr,
+"(It is not the codename of any distribution and there are multiple\n"
+"distributions with this as suite name.)\n");
 			return RET_ERROR_MISSING;
 		}
 	}
-	free(found);
 	return RET_OK;
 }
 
 retvalue distribution_get(struct distribution *alldistributions, const char *name, bool lookedat, struct distribution **distribution) {
-	struct distribution *d;
+	struct distribution *d, *d2;
 
 	d = alldistributions;
 	while( d != NULL && strcmp(name, d->codename) != 0 )
 		d = d->next;
 	if( d == NULL ) {
-		fprintf(stderr,"Cannot find definition of distribution '%s'!\n", name);
+		for( d2 = alldistributions; d2 != NULL ; d2 = d2->next ) {
+			if( d2->suite == NULL )
+				continue;
+			if( strcmp(name, d2->suite) != 0 )
+				continue;
+			if( d != NULL ) {
+				fprintf(stderr,
+"No distribution has '%s' as codename, but multiple as suite name,\n"
+"thus it cannot be used to determine a distribution.\n", name);
+				return RET_ERROR_MISSING;
+			}
+			d = d2;
+		}
+	}
+	if( d == NULL ) {
+		fprintf(stderr, "Cannot find definition of distribution '%s'!\n", name);
 		return RET_ERROR_MISSING;
 	}
 	d->selected = true;
@@ -671,7 +650,7 @@ retvalue distribution_get(struct distribution *alldistributions, const char *nam
 	return RET_OK;
 }
 
-retvalue distribution_snapshot(struct distribution *distribution, const char *distdir, struct database *database, const char *name) {
+retvalue distribution_snapshot(struct distribution *distribution, struct database *database, const char *name) {
 	struct target *target;
 	retvalue result,r;
 	struct release *release;
@@ -679,7 +658,7 @@ retvalue distribution_snapshot(struct distribution *distribution, const char *di
 
 	assert( distribution != NULL );
 
-	r = release_initsnapshot(distdir,distribution->codename,name,&release);
+	r = release_initsnapshot(distribution->codename, name, &release);
 	if( RET_WAS_ERROR(r) )
 		return r;
 
@@ -722,14 +701,16 @@ retvalue distribution_snapshot(struct distribution *distribution, const char *di
 	return result;
 }
 
-static retvalue export(struct distribution *distribution, const char *distdir, struct database *database, bool onlyneeded) {
+static retvalue export(struct distribution *distribution, struct database *database, bool onlyneeded) {
 	struct target *target;
 	retvalue result,r;
 	struct release *release;
 
 	assert( distribution != NULL );
 
-	r = release_init(&release, database, distdir, distribution->codename);
+	r = release_init(&release, database,
+			distribution->codename, distribution->suite,
+			distribution->fakecomponentprefix);
 	if( RET_WAS_ERROR(r) )
 		return r;
 
@@ -771,9 +752,9 @@ static retvalue export(struct distribution *distribution, const char *distdir, s
 		if( workleft ) {
 			(void)fputs(
 "This means that from outside your repository will still look like before (and\n"
-"should still work if this old state worked), but the things you changed in this\n"
+"should still work if this old state worked), but the changes intended with this\n"
 "call will not be visible until you call export directly (via reprepro export)\n"
-"(things will also get visible when something else changes the same file and\n"
+"Changes will also get visible when something else changes the same file and\n"
 "thus creates a new export of that file, but even changes to other parts of the\n"
 "same distribution will not!\n",	stderr);
 		}
@@ -786,8 +767,8 @@ static retvalue export(struct distribution *distribution, const char *distdir, s
 	return result;
 }
 
-retvalue distribution_fullexport(struct distribution *distribution, const char *distdir, struct database *database) {
-	return export(distribution, distdir, database, false);
+retvalue distribution_fullexport(struct distribution *distribution, struct database *database) {
+	return export(distribution, database, false);
 }
 
 retvalue distribution_freelist(struct distribution *distributions) {
@@ -803,7 +784,7 @@ retvalue distribution_freelist(struct distribution *distributions) {
 	return result;
 }
 
-retvalue distribution_exportlist(enum exportwhen when, struct distribution *distributions, const char *distdir, struct database *database) {
+retvalue distribution_exportlist(enum exportwhen when, struct distribution *distributions, struct database *database) {
 	retvalue result,r;
 	bool todo = false;
 	struct distribution *d;
@@ -859,7 +840,7 @@ retvalue distribution_exportlist(enum exportwhen when, struct distribution *dist
 "Please report this and how you got this message as bugreport. Thanks.\n"
 "Doing a export despite --export=changed....\n",
 						d->codename);
-					r = export(d, distdir, database, true);
+					r = export(d, database, true);
 					RET_UPDATE(result,r);
 					break;
 				}
@@ -869,14 +850,14 @@ retvalue distribution_exportlist(enum exportwhen when, struct distribution *dist
 					( d->status == RET_NOTHING &&
 					  when != EXPORT_CHANGED) ||
 					when == EXPORT_FORCE);
-			r = export(d, distdir, database, true);
+			r = export(d, database, true);
 			RET_UPDATE(result,r);
 		}
 	}
 	return result;
 }
 
-retvalue distribution_export(enum exportwhen when, struct distribution *distribution, const char *distdir, struct database *database) {
+retvalue distribution_export(enum exportwhen when, struct distribution *distribution, struct database *database) {
 	if( when == EXPORT_NEVER ) {
 		if( verbose >= 10 )
 			fprintf(stderr,
@@ -909,8 +890,7 @@ retvalue distribution_export(enum exportwhen when, struct distribution *distribu
 "Please report this and how you got this message as bugreport. Thanks.\n"
 "Doing a export despite --export=changed....\n",
 						distribution->codename);
-				return export(distribution, distdir,
-						database, true);
+				return export(distribution, database, true);
 				break;
 			}
 		}
@@ -919,7 +899,7 @@ retvalue distribution_export(enum exportwhen when, struct distribution *distribu
 	}
 	if( verbose >= 0 )
 		printf("Exporting indices...\n");
-	return export(distribution, distdir, database, true);
+	return export(distribution, database, true);
 }
 
 /* get a pointer to the apropiate part of the linked list */
@@ -958,25 +938,25 @@ struct distribution *distribution_find(struct distribution *distributions, const
 	return NULL;
 }
 
-retvalue distribution_loadalloverrides(struct distribution *distribution, const char *confdir, const char *overridedir) {
+retvalue distribution_loadalloverrides(struct distribution *distribution) {
 	retvalue r;
 
 	if( distribution->overrides.deb == NULL ) {
-		r = override_read(confdir, overridedir, distribution->deb_override, &distribution->overrides.deb);
+		r = override_read(distribution->deb_override, &distribution->overrides.deb);
 		if( RET_WAS_ERROR(r) ) {
 			distribution->overrides.deb = NULL;
 			return r;
 		}
 	}
 	if( distribution->overrides.udeb == NULL ) {
-		r = override_read(confdir, overridedir, distribution->udeb_override, &distribution->overrides.udeb);
+		r = override_read(distribution->udeb_override, &distribution->overrides.udeb);
 		if( RET_WAS_ERROR(r) ) {
 			distribution->overrides.udeb = NULL;
 			return r;
 		}
 	}
 	if( distribution->overrides.dsc == NULL ) {
-		r = override_read(confdir, overridedir, distribution->dsc_override, &distribution->overrides.dsc);
+		r = override_read(distribution->dsc_override, &distribution->overrides.dsc);
 		if( RET_WAS_ERROR(r) ) {
 			distribution->overrides.dsc = NULL;
 			return r;
@@ -990,12 +970,12 @@ retvalue distribution_loadalloverrides(struct distribution *distribution, const
 		return RET_NOTHING;
 }
 
-retvalue distribution_loaduploaders(struct distribution *distribution, const char *confdir) {
+retvalue distribution_loaduploaders(struct distribution *distribution) {
 	if( distribution->uploaders != NULL ) {
 		if( distribution->uploaderslist != NULL )
 			return RET_OK;
 		return uploaders_get(&distribution->uploaderslist,
-				confdir, distribution->uploaders);
+				distribution->uploaders);
 	} else {
 		distribution->uploaderslist = NULL;
 		return RET_NOTHING;
@@ -1025,7 +1005,7 @@ retvalue distribution_prepareforwriting(struct distribution *distribution) {
 retvalue distribution_remove_packages(struct distribution *distribution, struct database *database, const char *component, const char *architecture, const char *packagetype, each_package_action decider, struct strlist *dereferenced, struct trackingdata *trackingdata, void *data) {
 	retvalue result,r;
 	struct target *t;
-	struct cursor *cursor;
+	struct target_cursor iterator;
 	const char *package, *control;
 
 	result = RET_NOTHING;
@@ -1036,35 +1016,25 @@ retvalue distribution_remove_packages(struct distribution *distribution, struct
 			continue;
 		if( packagetype != NULL && strcmp(packagetype,t->packagetype) != 0 )
 			continue;
-		r = target_initpackagesdb(t, database, READWRITE);
+		r = target_openiterator(t, database, READWRITE, &iterator);
 		RET_UPDATE(result, r);
 		if( RET_WAS_ERROR(r) )
 			return result;
-		r = table_newglobalcursor(t->packages, &cursor);
-		assert( r != RET_NOTHING );
-		if( RET_WAS_ERROR(r) ) {
-			(void)target_closepackagesdb(t);
-			return r;
-		}
-		while( cursor_nexttemp(t->packages, cursor,
-					&package, &control) ) {
+		while( target_nextpackage(&iterator, &package, &control) ) {
 			r = decider(database, distribution, t,
 					package, control, data);
 			RET_UPDATE(result, r);
 			if( RET_WAS_ERROR(r) )
 				break;
 			if( RET_IS_OK(r) ) {
-				r = target_removepackage_by_cursor(t,
-					distribution->logger, database, cursor,
-					package, control, NULL, dereferenced,
-					trackingdata);
+				r = target_removepackage_by_cursor(&iterator,
+					distribution->logger, database,
+					dereferenced, trackingdata);
 				RET_UPDATE(result, r);
 				RET_UPDATE(distribution->status, r);
 			}
 		}
-		r = cursor_close(t->packages, cursor);
-		RET_ENDUPDATE(result, r);
-		r = target_closepackagesdb(t);
+		r = target_closeiterator(&iterator);
 		RET_ENDUPDATE(result, r);
 		if( RET_WAS_ERROR(result) )
 			return result;
