@@ -55,7 +55,7 @@ static retvalue try_extractcontrol(char **control, const char *debfile, bool bro
 	ret = pipe(pipe1);
 	if( ret < 0 ) {
 		int e = errno;
-		fprintf(stderr, "Error while creating pipe: %d=%s\n",e,strerror(e));
+		fprintf(stderr, "Error %d creating pipe: %s\n", e, strerror(e));
 		return RET_ERRNO(e);
 	}
 
@@ -63,14 +63,14 @@ static retvalue try_extractcontrol(char **control, const char *debfile, bool bro
 	if( ret < 0 ) {
 		int e = errno;
 		close(pipe1[0]);close(pipe1[1]);
-		fprintf(stderr, "Error while creating pipe: %d=%s\n",e,strerror(e));
+		fprintf(stderr, "Error %d creating pipe: %s\n", e, strerror(e));
 		return RET_ERRNO(e);
 	}
 
 	ar = fork();
 	if( ar < 0 ) {
 		int e = errno;
-		fprintf(stderr, "Error while forking: %d=%s\n",e,strerror(e));
+		fprintf(stderr, "Error %d forking: %s\n", e, strerror(e));
 		result = RET_ERRNO(e);
 		close(pipe1[0]);close(pipe1[1]);
 		close(pipe2[0]);close(pipe2[1]);
@@ -78,14 +78,19 @@ static retvalue try_extractcontrol(char **control, const char *debfile, bool bro
 	}
 
 	if( ar == 0 ) {
+		int e;
 		/* calling ar */
 		if( dup2(pipe1[1],1) < 0 )
 			exit(255);
 		close(pipe1[0]);close(pipe1[1]);
 		close(pipe2[0]);close(pipe2[1]);
 		//TODO without explicit path
-		ret = execl("/usr/bin/ar","ar","p",debfile,"control.tar.gz",NULL);
-		fprintf(stderr,"calling ar failed: %m\n");
+		ret = execl("/usr/bin/ar",
+				"ar", "p", debfile, "control.tar.gz",
+				ENDOFARGUMENTS);
+		e = errno;
+		fprintf(stderr, "ar call failed with error %d: %s\n",
+				e, strerror(e));
 		exit(254);
 	}
 
@@ -93,11 +98,12 @@ static retvalue try_extractcontrol(char **control, const char *debfile, bool bro
 	if( tar < 0 ) {
 		int e = errno;
 		result = RET_ERRNO(e);
-		fprintf(stderr, "Error while forking: %d=%s\n",e,strerror(e));
+		fprintf(stderr, "Error %d forking: %s\n", e, strerror(e));
 		close(pipe1[0]);close(pipe1[1]);
 		close(pipe2[0]);close(pipe2[1]);
 		tar = -1;
 	} else if( tar == 0 ) {
+		int e;
 		/* calling tar */
 		if( dup2(pipe1[0],0) < 0 )
 			exit(255);
@@ -107,8 +113,11 @@ static retvalue try_extractcontrol(char **control, const char *debfile, bool bro
 		close(pipe2[0]);close(pipe2[1]);
 		//TODO without explicit path
 		execl("/bin/tar", "tar", "-xOzf", "-",
-				brokentar?"control":"./control", NULL);
-		fprintf(stderr,"calling tar failed: %m\n");
+				brokentar?"control":"./control",
+				ENDOFARGUMENTS);
+		e = errno;
+		fprintf(stderr, "tar call failed with error %d: %s\n",
+				e, strerror(e));
 		exit(254);
 
 	}
@@ -145,7 +154,7 @@ static retvalue try_extractcontrol(char **control, const char *debfile, bool bro
 		if( r == RET_NOTHING ) {
 			free(controlchunk);
 			controlchunk = NULL;
-			fprintf(stderr,"Got no control information from .deb!\n");
+			fprintf(stderr, "No control information found in .deb!\n");
 			/* only report error now if we haven't try everything yet */
 			if( brokentar )
 				r = RET_ERROR_MISSING;
@@ -220,7 +229,7 @@ retvalue extractcontrol(char **control,const char *debfile) {
 }
 
 retvalue getfilelist(/*@out@*/char **filelist, /*@out@*/size_t *size, const char *debfile) {
-	fprintf(stderr, "Extracing of filelist currently not implemented without libarchive.\n");
+	fprintf(stderr, "Extraction of file list without libarchive currently not implemented.\n");
 	return RET_ERROR;
 #if 0
 	int pipe1[2];
@@ -242,7 +251,7 @@ retvalue getfilelist(/*@out@*/char **filelist, /*@out@*/size_t *size, const char
 	ret = pipe(pipe1);
 	if( ret < 0 ) {
 		int e = errno;
-		fprintf(stderr, "Error while creating pipe: %d=%s\n",e,strerror(e));
+		fprintf(stderr, "Error %d creating pipe: %s\n", e, strerror(e));
 		filelistcompressor_cancel(&c);
 		return RET_ERRNO(e);
 	}
@@ -250,7 +259,7 @@ retvalue getfilelist(/*@out@*/char **filelist, /*@out@*/size_t *size, const char
 	ret = pipe(pipe2);
 	if( ret < 0 ) {
 		int e = errno;
-		fprintf(stderr, "Error while creating pipe: %d=%s\n",e,strerror(e));
+		fprintf(stderr, "Error %d creating pipe: %s\n", e, strerror(e));
 		close(pipe1[0]);close(pipe1[1]);
 		filelistcompressor_cancel(&c);
 		return RET_ERRNO(e);
@@ -259,7 +268,7 @@ retvalue getfilelist(/*@out@*/char **filelist, /*@out@*/size_t *size, const char
 	ar = fork();
 	if( ar < 0 ) {
 		int e = errno;
-		fprintf(stderr, "Error while forking: %d=%s\n",e,strerror(e));
+		fprintf(stderr, "Error %d forking: %s\n", e, strerror(e));
 		result = RET_ERRNO(e);
 		close(pipe1[0]);close(pipe1[1]);
 		close(pipe2[0]);close(pipe2[1]);
@@ -268,14 +277,19 @@ retvalue getfilelist(/*@out@*/char **filelist, /*@out@*/size_t *size, const char
 	}
 
 	if( ar == 0 ) {
+		int e;
 		/* calling ar */
 		if( dup2(pipe1[1],1) < 0 )
 			exit(255);
 		close(pipe1[0]);close(pipe1[1]);
 		close(pipe2[0]);close(pipe2[1]);
 		//TODO without explicit path
-		ret = execl("/usr/bin/ar","ar","p",debfile,"data.tar.gz",NULL);
-		fprintf(stderr,"calling ar failed: %m\n");
+		ret = execl("/usr/bin/ar",
+				"ar", "p", debfile, "data.tar.gz",
+				ENDOFARGUMENTS);
+		e = errno;
+		fprintf(stderr, "ar call failed with error %d: %s\n",
+				e, strerror(e));
 		exit(254);
 	}
 
@@ -283,11 +297,12 @@ retvalue getfilelist(/*@out@*/char **filelist, /*@out@*/size_t *size, const char
 	if( tar < 0 ) {
 		int e = errno;
 		result = RET_ERRNO(e);
-		fprintf(stderr, "Error while forking: %d=%s\n",e,strerror(e));
+		fprintf(stderr, "Error %d forking: %s\n", e, strerror(e));
 		close(pipe1[0]);close(pipe1[1]);
 		close(pipe2[0]);close(pipe2[1]);
 		tar = -1;
 	} else if( tar == 0 ) {
+		int e;
 		/* calling tar */
 		if( dup2(pipe1[0],0) < 0 )
 			exit(255);
@@ -296,8 +311,10 @@ retvalue getfilelist(/*@out@*/char **filelist, /*@out@*/size_t *size, const char
 		close(pipe1[0]);close(pipe1[1]);
 		close(pipe2[0]);close(pipe2[1]);
 		//TODO without explicit path
-		execl("/bin/tar","tar","-tzf","-",NULL);
-		fprintf(stderr,"calling tar failed: %m\n");
+		execl("/bin/tar", "tar", "-tzf", "-", ENDOFARGUMENTS);
+		e = errno;
+		fprintf(stderr, "tar call failed with error %d: %s\n",
+				e, strerror(e));
 		exit(254);
 
 	}
@@ -326,7 +343,7 @@ retvalue getfilelist(/*@out@*/char **filelist, /*@out@*/size_t *size, const char
 		bytes_read = read(pipe2[0], list+len, listsize-len-1);
 		if( bytes_read < 0 ) {
 			int e = errno;
-			fprintf(stderr, "Error reading from pipe: %d=%s\n",
+			fprintf(stderr, "Error %d reading from pipe: %s\n",
 					e, strerror(e));
 			result = RET_ERRNO(e);
 			break;
@@ -334,7 +351,8 @@ retvalue getfilelist(/*@out@*/char **filelist, /*@out@*/size_t *size, const char
 			break;
 		else while( bytes_read > 0 ) {
 			if( list[len] == '\0' ) {
-				fprintf(stderr, "Unexpected NUL character from tar while getting filelist from %s!\n",debfile);
+				fprintf(stderr,
+"Unexpected NUL character from tar while getting file list from %s!\n", debfile);
 				result = RET_ERROR;
 				break;
 			} else if( list[len] == '\n' ) {
@@ -372,7 +390,8 @@ retvalue getfilelist(/*@out@*/char **filelist, /*@out@*/size_t *size, const char
 		}
 	} while( true );
 	if( len != last ) {
-		fprintf(stderr, "WARNING: unterminated output from tar over pipe while extracting filelist of %s\n",debfile);
+		fprintf(stderr,
+"WARNING: unterminated output from tar pipe while extracting file list of %s\n", debfile);
 		list[len] = '\0';
 		fprintf(stderr, "The item '%s' might got lost.\n",
 				list+last);
