@@ -16,6 +16,7 @@
 #include <config.h>
 
 #include <malloc.h>
+#include <stdlib.h>
 #include <assert.h>
 #include <string.h>
 #include <stdio.h>
@@ -428,7 +429,7 @@ static retvalue filelist_addfiles(struct filelist_list *list, const struct filel
 }
 
 retvalue filelist_addpackage(struct filelist_list *list, struct database *database, const char *packagename, const char *section, const char *filekey) {
-	const struct filelist_package *package;
+	const struct filelist_package *package IFSTUPIDCC(=NULL);
 	char *debfilename, *contents = NULL;
 	retvalue r;
 	const char *c;
@@ -568,7 +569,7 @@ static inline bool filelistcompressor_space(struct filelistcompressor *c, size_t
 		char *n;
 
 		if( c->size > 1024*1024*1024 ) {
-			fprintf(stderr, "Ridicilous long filelist!\n");
+			fprintf(stderr, "Ridiculously long file list!\n");
 			return false;
 		}
 		c->size = c->len + len + 2048;
