@@ -117,14 +117,14 @@ retvalue files_remove(struct database *database, const char *filekey, bool ignor
 		(void)table_deleterecord(database->checksums, filekey, true);
 		r = table_deleterecord(database->oldmd5sums, filekey, true);
 		if( r == RET_NOTHING && !ignoremissing ) {
-			fprintf(stderr, "To be forgotten filekey '%s' was not known.\n",
+			fprintf(stderr, "Unable to forget unknown filekey '%s'.\n",
 					filekey);
 			return RET_ERROR_MISSING;
 		}
 	} else {
 		r = table_deleterecord(database->checksums, filekey, true);
 		if( r == RET_NOTHING && !ignoremissing ) {
-			fprintf(stderr, "To be forgotten filekey '%s' was not known.\n",
+			fprintf(stderr, "Unable to forget unknown filekey '%s'.\n",
 					filekey);
 			return RET_ERROR_MISSING;
 		}
@@ -153,8 +153,8 @@ retvalue files_deleteandremove(struct database *database, const char *filekey, b
 			if( !ignoreifnot )
 				fprintf(stderr,"%s not found, forgetting anyway\n",filename);
 		} else {
-			fprintf(stderr, "error while unlinking %s: %m(%d)\n",
-					filename, en);
+			fprintf(stderr, "error %d while unlinking %s: %s\n",
+					en, filename, strerror(en));
 			free(filename);
 			return r;
 		}
@@ -184,7 +184,8 @@ retvalue files_deleteandremove(struct database *database, const char *filekey, b
 					//other error was first and it
 					//is not empty so we do not have
 					//to remove it anyway...
-					fprintf(stderr,"ignoring error trying to rmdir %s: %m(%d)\n",filename,en);
+					fprintf(stderr,
+"ignoring error %d trying to rmdir %s: %s\n", en, filename, strerror(en));
 				}
 				/* parent directories will contain this one
 				 * thus not be empty, in other words:
@@ -267,7 +268,7 @@ retvalue files_expect(struct database *database, const char *filekey, const stru
 	if( r == RET_ERROR_WRONG_MD5) {
 		fprintf(stderr,
 "Deleting unexpected file '%s'!\n"
-"(found in pool but not in the database and file size is wrong.)\n ",
+"(found in pool but not in database and file size is wrong.)\n ",
 				filename);
 		if( unlink(filename) == 0 )
 			r = RET_NOTHING;
@@ -763,7 +764,7 @@ static retvalue collectnewchecksums(struct database *database, const char *filek
 "wrong. I suggest comparing the output of _listchecksums with the files\n"
 "actually found in the pool.\n",	filekey);
 		if( improves ) {
-			r = checksums_combine(&expected, real);
+			r = checksums_combine(&expected, real, NULL);
 			assert( r != RET_NOTHING );
 			if( RET_WAS_ERROR(r) ) {
 				checksums_free(expected);
@@ -838,7 +839,7 @@ retvalue files_collectnewchecksums(struct database *database) {
 				checksums_free(expected);
 				break;
 			}
-			r = checksums_complete(&expected, fullfilename);
+			r = checksums_complete(&expected, fullfilename, NULL);
 			if( r == RET_NOTHING ) {
 				fprintf(stderr,"Missing file '%s'!\n", fullfilename);
 				r = RET_ERROR_MISSING;
@@ -985,7 +986,7 @@ retvalue files_preinclude(struct database *database, const char *sourcefilename,
 			return RET_ERROR_WRONG_MD5;
 		}
 		if( improves ) {
-			r = checksums_combine(&checksums, realchecksums);
+			r = checksums_combine(&checksums, realchecksums, NULL);
 			if( RET_WAS_ERROR(r) ) {
 				checksums_free(realchecksums);
 				checksums_free(checksums);
@@ -1118,7 +1119,7 @@ static retvalue checkimproveorinclude(struct database *database, const char *sou
 		r = RET_ERROR_WRONG_MD5;
 	}
 	if( improves ) {
-		r = checksums_combine(checksums_p, checksums);
+		r = checksums_combine(checksums_p, checksums, NULL);
 		if( RET_IS_OK(r) )
 			*improving = true;
 	}
@@ -1164,7 +1165,7 @@ retvalue files_checkincludefile(struct database *database, const char *sourcedir
 		}
 		r = RET_NOTHING;
 		if( improves )
-			r = checksums_combine(&checksums, *checksums_p);
+			r = checksums_combine(&checksums, *checksums_p, NULL);
 		if( !RET_WAS_ERROR(r) )
 			r = checkimproveorinclude(database, sourcedir,
 				basename, filekey, &checksums, &improves);
@@ -1218,7 +1219,7 @@ retvalue files_checkincludefile(struct database *database, const char *sourcedir
 		return r;
 	}
 	if( improves ) {
-		r = checksums_combine(checksums_p, checksums);
+		r = checksums_combine(checksums_p, checksums, NULL);
 		checksums_free(checksums);
 		if( RET_WAS_ERROR(r) )
 			return r;
