@@ -144,7 +144,7 @@ static retvalue incoming_prepare(struct incoming *i) {
 	ret = closedir(dir);
 	if( ret != 0 ) {
 		int e = errno;
-		fprintf(stderr, "Error scaning '%s': %s\n", i->directory, strerror(e));
+		fprintf(stderr, "Error scanning '%s': %s\n", i->directory, strerror(e));
 		return RET_ERRNO(e);
 	}
 	i->processed = calloc(i->files.count, sizeof(bool));
@@ -160,7 +160,6 @@ struct read_incoming_data {
 	/*@temp@*/const char *name;
 	/*@temp@*/struct distribution *distributions;
 	struct incoming *i;
-	/*@temp@*/const char *basedir;
 };
 
 static retvalue translate(struct distribution *distributions, struct strlist *names, struct distribution ***r) {
@@ -211,7 +210,7 @@ CFfinishparse(incoming) {
 		return RET_ERROR;
 	}
 	if( i->tempdir[0] != '/' ) {
-		char *n = calc_dirconcat(d->basedir, i->tempdir);
+		char *n = calc_dirconcat(global.basedir, i->tempdir);
 		if( n == NULL ) {
 			incoming_free(i);
 			return RET_ERROR_OOM;
@@ -220,7 +219,7 @@ CFfinishparse(incoming) {
 		i->tempdir = n;
 	}
 	if( i->directory[0] != '/' ) {
-		char *n = calc_dirconcat(d->basedir, i->directory);
+		char *n = calc_dirconcat(global.basedir, i->directory);
 		if( n == NULL ) {
 			incoming_free(i);
 			return RET_ERROR_OOM;
@@ -230,7 +229,7 @@ CFfinishparse(incoming) {
 	}
 	if( i->default_into == NULL && i->allow.count == 0 ) {
 		fprintf(stderr,
-"There ia neither a 'Allow' nor a 'Default' definition in rule '%s'\n"
+"There is neither an 'Allow' nor a 'Default' definition in rule '%s'\n"
 "(starting at line %u, ending at line %u of %s)!\n"
 "Aborting as nothing would be let in.\n",
 				d->name,
@@ -352,16 +351,15 @@ static const struct configfield incomingconfigfields[] = {
 	CF("Permit", incoming, permit)
 };
 
-static retvalue incoming_init(const char *basedir,const char *confdir, struct distribution *distributions, const char *name, /*@out@*/struct incoming **result) {
+static retvalue incoming_init(struct distribution *distributions, const char *name, /*@out@*/struct incoming **result) {
 	retvalue r;
 	struct read_incoming_data imports;
 
 	imports.name = name;
 	imports.distributions = distributions;
 	imports.i = NULL;
-	imports.basedir = basedir;
 
-	r = configfile_parse(confdir, "incoming", IGNORABLE(unknownfield),
+	r = configfile_parse("incoming", IGNORABLE(unknownfield),
 			startparseincoming, finishparseincoming,
 			incomingconfigfields, ARRAYCOUNT(incomingconfigfields),
 			&imports);
@@ -369,7 +367,7 @@ static retvalue incoming_init(const char *basedir,const char *confdir, struct di
 		return r;
 	if( imports.i == NULL ) {
 		fprintf(stderr, "No definition for '%s' found in '%s/incoming'!\n",
-				name, confdir);
+				name, global.confdir);
 		return RET_ERROR_MISSING;
 	}
 
@@ -386,7 +384,7 @@ struct candidate {
 	/* from candidate_read */
 	int ofs;
 	char *control;
-	struct strlist keys;
+	struct strlist keys, allkeys;
 	/* from candidate_parse */
 	char *source, *sourceversion, *changesversion;
 	struct strlist distributions,
@@ -414,6 +412,8 @@ struct candidate {
 		struct deb_headers deb;
 		/* - only for fe_DSC types */
 		struct dsc_headers dsc;
+		/* only valid while parsing */
+		struct hashes h;
 	} *files;
 	struct candidate_perdistribution {
 		struct candidate_perdistribution *next;
@@ -472,6 +472,7 @@ static void candidate_free(/*@only@*/struct candidate *c) {
 		return;
 	free(c->control);
 	strlist_done(&c->keys);
+	strlist_done(&c->allkeys);
 	free(c->source);
 	free(c->sourceversion);
 	free(c->changesversion);
@@ -551,7 +552,7 @@ static retvalue candidate_read(struct incoming *i, int ofs, struct candidate **r
 		return r;
 	}
 	assert( n->files->tempfilename != NULL );
-	r = signature_readsignedchunk(n->files->tempfilename, BASENAME(i,ofs), &n->control, &n->keys, NULL, broken);
+	r = signature_readsignedchunk(n->files->tempfilename, BASENAME(i,ofs), &n->control, &n->keys, &n->allkeys, broken);
 	assert( r != RET_NOTHING );
 	if( RET_WAS_ERROR(r) ) {
 		candidate_free(n);
@@ -570,7 +571,8 @@ static retvalue candidate_addfileline(struct incoming *i, struct candidate *c, c
 	if( n == NULL )
 		return RET_ERROR_OOM;
 
-	r = changes_parsefileline(fileline, &n->type, &basename, &n->checksums,
+	r = changes_parsefileline(fileline, &n->type, &basename,
+			&n->h.hashes[cs_md5sum], &n->h.hashes[cs_length],
 			&n->section, &n->priority, &n->architecture, &n->name);
 	if( RET_WAS_ERROR(r) ) {
 		free(n);
@@ -592,14 +594,69 @@ static retvalue candidate_addfileline(struct incoming *i, struct candidate *c, c
 	return RET_OK;
 }
 
+static retvalue candidate_addhashes(struct incoming *i, struct candidate *c, enum checksumtype cs, const struct strlist *lines) {
+	int j;
+
+	for( j = 0 ; j < lines->count ; j++ ) {
+		const char *fileline = lines->values[j];
+		struct candidate_file *f;
+		const char *basename;
+		struct hash_data hash, size;
+		retvalue r;
+
+		r = hashline_parse(BASENAME(i, c->ofs), fileline, cs,
+				&basename, &hash, &size);
+		if( !RET_IS_OK(r) )
+			return r;
+		f = c->files;
+		while( f != NULL && strcmp(BASENAME(i, f->ofs), basename) != 0 )
+			f = f->next;
+		if( f == NULL ) {
+			fprintf(stderr,
+"Warning: Ignoring file '%s' listed in '%s' but not in '%s' of '%s'!\n",
+					basename, changes_checksum_names[cs],
+					changes_checksum_names[cs_md5sum],
+					BASENAME(i, c->ofs));
+			continue;
+		}
+		if( f->h.hashes[cs_length].len != size.len ||
+				memcmp(f->h.hashes[cs_length].start,
+					size.start, size.len) != 0 ) {
+			fprintf(stderr,
+"Error: Different size of '%s' listed in '%s' and '%s' of '%s'!\n",
+					basename, changes_checksum_names[cs],
+					changes_checksum_names[cs_md5sum],
+					BASENAME(i, c->ofs));
+			return RET_ERROR;
+		}
+		f->h.hashes[cs] = hash;
+	}
+	return RET_OK;
+}
+
+static retvalue candidate_finalizechecksums(struct candidate *c) {
+	struct candidate_file *f;
+	retvalue r;
+
+	/* store collected hashes as checksums structs,
+	 * starting after .changes file: */
+	for( f = c->files->next ; f != NULL ; f = f->next ) {
+		r = checksums_initialize(&f->checksums, f->h.hashes);
+		if( RET_WAS_ERROR(r) )
+			return r;
+	}
+	return RET_OK;
+}
+
 static retvalue candidate_parse(struct incoming *i, struct candidate *c) {
 	retvalue r;
-	struct strlist filelines;
+	struct strlist filelines[cs_hashCOUNT];
+	enum checksumtype cs;
 	int j;
 #define R if( RET_WAS_ERROR(r) ) return r;
-#define E(err) { \
+#define E(err,...) { \
 		if( r == RET_NOTHING ) { \
-			fprintf(stderr,"In '%s': " err "\n",BASENAME(i,c->ofs)); \
+			fprintf(stderr, "In '%s': " err "\n",BASENAME(i,c->ofs), ## __VA_ARGS__ ); \
 			r = RET_ERROR; \
 	  	} \
 		if( RET_WAS_ERROR(r) ) return r; \
@@ -635,16 +692,36 @@ static retvalue candidate_parse(struct incoming *i, struct candidate *c) {
 	}
 	r = chunk_getwordlist(c->control,"Distribution",&c->distributions);
 	E("Missing 'Distribution' field!");
-	r = chunk_getextralinelist(c->control,"Files",&filelines);
-	E("Missing 'Files' field!");
-	for( j = 0 ; j < filelines.count ; j++ ) {
-		r = candidate_addfileline(i, c, filelines.values[j]);
+	r = chunk_getextralinelist(c->control,
+			changes_checksum_names[cs_md5sum],
+			&filelines[cs_md5sum]);
+	E("Missing '%s' field!", changes_checksum_names[cs_md5sum]);
+	for( j = 0 ; j < filelines[cs_md5sum].count ; j++ ) {
+		r = candidate_addfileline(i, c, filelines[cs_md5sum].values[j]);
 		if( RET_WAS_ERROR(r) ) {
-			strlist_done(&filelines);
+			strlist_done(&filelines[cs_md5sum]);
 			return r;
 		}
 	}
-	strlist_done(&filelines);
+	for( cs = cs_firstEXTENDED ; cs < cs_hashCOUNT ; cs++ ) {
+		r = chunk_getextralinelist(c->control,
+				changes_checksum_names[cs], &filelines[cs]);
+
+		if( RET_IS_OK(r) )
+			r = candidate_addhashes(i, c, cs, &filelines[cs]);
+		else
+			strlist_init(&filelines[cs]);
+
+		if( RET_WAS_ERROR(r) ) {
+			while( cs-- > cs_md5sum )
+				strlist_done(&filelines[cs]);
+			return r;
+		}
+	}
+	r = candidate_finalizechecksums(c);
+	for( cs = cs_md5sum ; cs < cs_hashCOUNT ; cs++ )
+		strlist_done(&filelines[cs]);
+	R;
 	if( c->files == NULL || c->files->next == NULL ) {
 		fprintf(stderr,"In '%s': Empty 'Files' section!\n",
 				BASENAME(i,c->ofs));
@@ -727,7 +804,7 @@ static retvalue candidate_usefile(const struct incoming *i,const struct candidat
 		return RET_ERROR_WRONG_MD5;
 	}
 	if( improves ) {
-		r = checksums_combine(&file->checksums, readchecksums);
+		r = checksums_combine(&file->checksums, readchecksums, NULL);
 		if( RET_WAS_ERROR(r) ) {
 			checksums_free(readchecksums);
 			deletefile(tempfilename);
@@ -801,7 +878,7 @@ static retvalue candidate_read_deb(struct incoming *i,struct candidate *c,struct
 	if( strcmp(c->source, file->deb.source) != 0 ) {
 		// TODO: add permissive thing to ignore this
 		// (beware if tracking is active)
-		fprintf(stderr, "Source-header '%s' of '%s' and source name '%s' within the file '%s' do not match!\n",
+		fprintf(stderr, "Source header '%s' of '%s' and source name '%s' within the file '%s' do not match!\n",
 				c->source, BASENAME(i,c->ofs),
 				file->deb.source, BASENAME(i,file->ofs));
 		return RET_ERROR;
@@ -815,7 +892,7 @@ static retvalue candidate_read_deb(struct incoming *i,struct candidate *c,struct
 		return RET_ERROR;
 	}
 	if( ! strlist_in(&c->binaries, file->deb.name) ) {
-		fprintf(stderr, "Name '%s' of binary '%s' is not listed in Binaries-header of '%s'!\n",
+		fprintf(stderr, "Name '%s' of binary '%s' is not listed in Binaries header of '%s'!\n",
 				file->deb.name, BASENAME(i,file->ofs),
 				BASENAME(i,c->ofs));
 		return RET_ERROR;
@@ -940,7 +1017,7 @@ static retvalue candidate_preparechangesfile(struct database *database,const str
 }
 
 static retvalue prepare_deb(struct database *database,const struct incoming *i,const struct candidate *c,struct candidate_perdistribution *per,const struct candidate_file *file) {
-	const char *section,*priority, *filekey;
+	const char *section IFSTUPIDCC(=NULL), *priority IFSTUPIDCC(=NULL), *filekey;
 	const struct overrideinfo *oinfo;
 	struct candidate_package *package;
 	const struct distribution *into = per->into;
@@ -996,9 +1073,11 @@ static retvalue prepare_deb(struct database *database,const struct incoming *i,c
 	return RET_OK;
 }
 
-static retvalue prepare_source_file(struct database *database, const struct incoming *i, const struct candidate *c, const char *filekey, const char *basename, const struct checksums *checksums, int package_ofs, /*@out@*/const struct candidate_file **foundfile_p){
+static retvalue prepare_source_file(struct database *database, const struct incoming *i, const struct candidate *c, const char *filekey, const char *basename, struct checksums **checksums_p, int package_ofs, /*@out@*/const struct candidate_file **foundfile_p){
 	struct candidate_file *f;
+	const struct checksums * const checksums = *checksums_p;
 	retvalue r;
+	bool improves;
 
 	f = c->files;
 	while( f != NULL && (f->checksums == NULL ||
@@ -1028,17 +1107,22 @@ static retvalue prepare_source_file(struct database *database, const struct inco
 		/* otherwise proceed with the found file: */
 	}
 
-	// TODO: is the error message sensible?
-	// perhaps currently, but in the future when .changes or .dsc
-	// can have additional hashes, they might not conflict with each
-	// other, but with the file's hash previously computed...
-	if( !checksums_check(f->checksums, checksums, NULL) ) {
+	if( !checksums_check(f->checksums, checksums, &improves) ) {
 		fprintf(stderr, "file '%s' has conflicting checksums listed in '%s' and '%s'!\n",
 				basename,
 				BASENAME(i, c->ofs),
 				BASENAME(i, package_ofs));
 		return RET_ERROR;
 	}
+	if( improves ) {
+		/* put additional checksums from the .dsc to the information
+		 * found in .changes, so that a file matching those in .changes
+		 * but not in .dsc is detected */
+		r = checksums_combine(&f->checksums, checksums, NULL);
+		assert( r != RET_NOTHING );
+		if( RET_WAS_ERROR(r) )
+			return r;
+	}
 	r = files_canadd(database, filekey, f->checksums);
 	if( r == RET_NOTHING ) {
 		/* already in the pool, mark as used (in the sense
@@ -1055,11 +1139,15 @@ static retvalue prepare_source_file(struct database *database, const struct inco
 		// TODO: update checksums to now received checksums?
 		*foundfile_p = f;
 	}
+	if( !RET_WAS_ERROR(r) && !checksums_iscomplete(checksums) ) {
+		/* update checksums so the source index can show them */
+		r = checksums_combine(checksums_p, f->checksums, NULL);
+	}
 	return r;
 }
 
 static retvalue prepare_dsc(struct database *database,const struct incoming *i,const struct candidate *c,struct candidate_perdistribution *per,const struct candidate_file *file) {
-	const char *section,*priority;
+	const char *section IFSTUPIDCC(=NULL), *priority IFSTUPIDCC(=NULL);
 	const struct overrideinfo *oinfo;
 	struct candidate_package *package;
 	const struct distribution *into = per->into;
@@ -1093,7 +1181,7 @@ static retvalue prepare_dsc(struct database *database,const struct incoming *i,c
 	if( strcmp(c->source, file->dsc.name) != 0 ) {
 		// TODO: add permissive thing to ignore this
 		// (beware if tracking is active)
-		fprintf(stderr, "Source-header '%s' of '%s' and name '%s' within the file '%s' do not match!\n",
+		fprintf(stderr, "Source header '%s' of '%s' and name '%s' within the file '%s' do not match!\n",
 				c->source, BASENAME(i,c->ofs),
 				file->dsc.name, BASENAME(i,file->ofs));
 		return RET_ERROR;
@@ -1138,7 +1226,7 @@ static retvalue prepare_dsc(struct database *database,const struct incoming *i,c
 		r = prepare_source_file(database, i, c,
 				package->filekeys.values[j],
 				file->dsc.files.names.values[j],
-				file->dsc.files.checksums[j],
+				&file->dsc.files.checksums[j],
 				file->ofs, &package->files[j]);
 		if( RET_WAS_ERROR(r) )
 			return r;
@@ -1306,6 +1394,7 @@ static retvalue candidate_add_into(struct database *database,struct strlist *der
 	struct distribution *into = d->into;
 	trackingdb tracks;
 	const char *changesfilekey = NULL;
+	char *origfilename;
 
 	if( interrupted() )
 		return RET_ERROR_INTERRUPTED;
@@ -1335,6 +1424,10 @@ static retvalue candidate_add_into(struct database *database,struct strlist *der
 		}
 	}
 
+	origfilename = calc_dirconcat(i->directory,
+			BASENAME(i, changesfile(c)->ofs));
+	causingfile = origfilename;
+
 	r = RET_OK;
 	for( p = d->packages ; p != NULL ; p = p->next ) {
 		if( p->skip ) {
@@ -1386,11 +1479,15 @@ static retvalue candidate_add_into(struct database *database,struct strlist *der
 		r2 = tracking_done(tracks);
 		RET_ENDUPDATE(r,r2);
 	}
-	if( RET_WAS_ERROR(r) )
+	if( RET_WAS_ERROR(r) ) {
+		free(origfilename);
 		return r;
+	}
 	logger_logchanges(into->logger, into->codename,
 			c->source, c->changesversion, c->control,
 			changesfile(c)->tempfilename, changesfilekey);
+	free(origfilename);
+	causingfile = NULL;
 	return RET_OK;
 }
 
@@ -1434,7 +1531,7 @@ static inline bool isallowed(UNUSED(struct incoming *i), UNUSED(struct candidate
 	return permissions->allowall;
 }
 
-static retvalue candidate_checkpermissions(const char *confdir, struct incoming *i, struct candidate *c, struct distribution *into) {
+static retvalue candidate_checkpermissions(struct incoming *i, struct candidate *c, struct distribution *into) {
 	retvalue r;
 	int j;
 
@@ -1442,7 +1539,7 @@ static retvalue candidate_checkpermissions(const char *confdir, struct incoming
 	if( into->uploaders == NULL )
 		return RET_OK;
 
-	r = distribution_loaduploaders(into, confdir);
+	r = distribution_loaduploaders(into);
 	if( RET_WAS_ERROR(r) )
 		return r;
 	assert(into->uploaderslist != NULL);
@@ -1453,9 +1550,8 @@ static retvalue candidate_checkpermissions(const char *confdir, struct incoming
 		r = uploaders_unsignedpermissions(into->uploaderslist,
 				&permissions);
 		assert( r != RET_NOTHING );
-		if( RET_WAS_ERROR(r) ) {
+		if( RET_WAS_ERROR(r) )
 			return r;
-		}
 		if( permissions != NULL && isallowed(i,c,into,permissions) )
 			return RET_OK;
 	} else for( j = 0; j < c->keys.count ; j++ ) {
@@ -1464,12 +1560,36 @@ static retvalue candidate_checkpermissions(const char *confdir, struct incoming
 		r = uploaders_permissions(into->uploaderslist,
 				c->keys.values[j], &permissions);
 		assert( r != RET_NOTHING );
-		if( RET_WAS_ERROR(r) ) {
+		if( RET_WAS_ERROR(r) )
 			return r;
-		}
 		if( permissions != NULL && isallowed(i,c,into,permissions) )
 			return RET_OK;
 	}
+	/* reject, check if it would have been accepted with more signatures
+	 * valid: */
+	if( verbose >= 0 &&
+	    c->allkeys.count != 0 && c->allkeys.count != c->keys.count ) {
+		for( j = 0; j < c->allkeys.count ; j++ ) {
+			const struct uploadpermissions *permissions;
+			r = uploaders_permissions(into->uploaderslist,
+					c->allkeys.values[j], &permissions);
+			assert( r != RET_NOTHING );
+			if( RET_WAS_ERROR(r) )
+				return r;
+			if( permissions != NULL &&
+			    isallowed(i,c , into, permissions) ) {
+				// TODO: get information if it was invalid because
+				// missing pub-key, expire-state or something else
+				// here so warning can be more specific.
+				fprintf(stderr,
+"'%s' would have been accepted into '%s' if signature with '%s' was checkable and valid.\n",
+					i->files.values[c->ofs],
+					into->codename,
+					c->allkeys.values[j]);
+			}
+		}
+	}
+
 	/* reject */
 	return RET_NOTHING;
 }
@@ -1516,7 +1636,7 @@ static retvalue check_architecture_availability(const struct incoming *i, const
 	return RET_OK;
 }
 
-static retvalue candidate_add(const char *confdir, const char *overridedir,struct database *database, struct strlist *dereferenced, struct incoming *i, struct candidate *c) {
+static retvalue candidate_add(struct database *database, struct strlist *dereferenced, struct incoming *i, struct candidate *c) {
 	struct candidate_perdistribution *d;
 	struct candidate_file *file;
 	retvalue r;
@@ -1530,7 +1650,7 @@ static retvalue candidate_add(const char *confdir, const char *overridedir,struc
 		return r;
 
 	for( d = c->perdistribution ; d != NULL ; d = d->next ) {
-		r = distribution_loadalloverrides(d->into, confdir, overridedir);
+		r = distribution_loadalloverrides(d->into);
 		if( RET_WAS_ERROR(r) )
 			return r;
 	}
@@ -1623,8 +1743,8 @@ static retvalue candidate_add(const char *confdir, const char *overridedir,struc
 	return RET_OK;
 }
 
-static retvalue process_changes(const char *confdir,const char *overridedir,struct database *database,struct strlist *dereferenced,struct incoming *i,int ofs) {
-	struct candidate *c;
+static retvalue process_changes(struct database *database, struct strlist *dereferenced, struct incoming *i, int ofs) {
+	struct candidate *c IFSTUPIDCC(=NULL);
 	struct candidate_file *file;
 	retvalue r;
 	int j,k;
@@ -1651,7 +1771,7 @@ static retvalue process_changes(const char *confdir,const char *overridedir,stru
 			// TODO: implement "*"
 			if( strcmp(name, i->allow.values[j]) == 0 ) {
 				tried = true;
-				r = candidate_checkpermissions(confdir, i, c,
+				r = candidate_checkpermissions(i, c,
 						i->allow_into[j]);
 				if( r == RET_NOTHING )
 					continue;
@@ -1671,7 +1791,7 @@ static retvalue process_changes(const char *confdir,const char *overridedir,stru
 	}
 	if( c->perdistribution == NULL && i->default_into != NULL ) {
 		tried = true;
-		r = candidate_checkpermissions(confdir, i, c, i->default_into);
+		r = candidate_checkpermissions(i, c, i->default_into);
 		if( RET_WAS_ERROR(r) ) {
 			candidate_free(c);
 			return r;
@@ -1702,7 +1822,7 @@ static retvalue process_changes(const char *confdir,const char *overridedir,stru
 				i->files.values[ofs]);
 			r = RET_ERROR;
 		} else
-			r = candidate_add(confdir, overridedir, database,
+			r = candidate_add(database,
 					dereferenced,
 					i, c);
 		if( RET_WAS_ERROR(r) && i->cleanup[cuf_on_error] ) {
@@ -1720,14 +1840,14 @@ static retvalue process_changes(const char *confdir,const char *overridedir,stru
 }
 
 /* tempdir should ideally be on the same partition like the pooldir */
-retvalue process_incoming(const char *basedir,const char *confdir,const char *overridedir,struct database *database,struct strlist *dereferenced,struct distribution *distributions,const char *name,const char *changesfilename) {
+retvalue process_incoming(struct database *database, struct strlist *dereferenced, struct distribution *distributions, const char *name, const char *changesfilename) {
 	struct incoming *i;
 	retvalue result,r;
 	int j;
 
 	result = RET_NOTHING;
 
-	r = incoming_init(basedir, confdir, distributions, name, &i);
+	r = incoming_init(distributions, name, &i);
 	if( RET_WAS_ERROR(r) )
 		return r;
 
@@ -1741,7 +1861,7 @@ retvalue process_incoming(const char *basedir,const char *confdir,const char *ov
 		if( changesfilename != NULL && strcmp(basename, changesfilename) != 0 )
 			continue;
 		/* a .changes file, check it */
-		r = process_changes(confdir, overridedir, database, dereferenced, i, j);
+		r = process_changes(database, dereferenced, i, j);
 		RET_UPDATE(result, r);
 	}
 
