@@ -42,6 +42,8 @@
 
 extern int verbose;
 
+const char *causingfile = NULL;
+
 /*@null@*/ static /*@refcounted@*/ struct logfile {
 	/*@null@*/struct logfile *next;
 	char *filename;
@@ -49,10 +51,10 @@ extern int verbose;
 	int fd;
 } *logfile_root = NULL;
 
-static retvalue logfile_reference(const char *logdir,/*@only@*/char *filename,/*@out@*/struct logfile **logfile) {
+static retvalue logfile_reference(/*@only@*/char *filename, /*@out@*/struct logfile **logfile) {
 	struct logfile *l;
 
-	assert( logdir != NULL && filename != NULL );
+	assert( global.logdir != NULL && filename != NULL );
 
 	for( l = logfile_root ; l != NULL ; l = l->next ) {
 		if( strcmp(l->filename, filename) == 0 ) {
@@ -70,7 +72,7 @@ static retvalue logfile_reference(const char *logdir,/*@only@*/char *filename,/*
 	if( filename[0] == '/' )
 		l->filename = filename;
 	else {
-		l->filename = calc_dirconcat(logdir,filename);
+		l->filename = calc_dirconcat(global.logdir, filename);
 		free(filename);
 	}
 	if( l->filename == NULL ) {
@@ -213,7 +215,7 @@ static void notificator_done(/*@special@*/struct notificator *n) /*@releases n->
 	free(n->architecture);
 }
 
-static retvalue notificator_parse(struct notificator *n, const char *confdir, struct configiterator *iter) {
+static retvalue notificator_parse(struct notificator *n, struct configiterator *iter) {
 	retvalue r;
 	int c;
 
@@ -367,7 +369,7 @@ static retvalue notificator_parse(struct notificator *n, const char *confdir, st
 			if( c != EOF ) {
 				fprintf(stderr,
 "Error parsing config file %s, line %u, column %u:\n"
-"Unexpected data at end of notifier after scriptname '%s'\n",
+"Unexpected data at end of notifier after script name '%s'\n",
 					config_filename(iter),
 					config_line(iter), config_column(iter),
 					script);
@@ -377,7 +379,7 @@ static retvalue notificator_parse(struct notificator *n, const char *confdir, st
 			if( script[0] == '/' )
 				n->scriptname = script;
 			else {
-				n->scriptname = calc_dirconcat(confdir, script);
+				n->scriptname = calc_conffile(script);
 				free(script);
 			}
 			if( n->scriptname == NULL )
@@ -395,6 +397,7 @@ static retvalue notificator_parse(struct notificator *n, const char *confdir, st
 /*@null@*/ static struct notification_process {
 	/*@null@*/struct notification_process *next;
 	char **arguments;
+	/*@null@*/char *causingfile;
 	/* data to send to the process */
 	size_t datalen, datasent;
 	/*@null@*/char *data;
@@ -410,6 +413,7 @@ static void notification_process_free(/*@only@*/struct notification_process *p)
 		(void)close(p->fd);
 	for( a = p->arguments ; *a != NULL ; a++ )
 		free(*a);
+	free(p->causingfile);
 	free(p->arguments);
 	free(p->data);
 	free(p);
@@ -452,7 +456,7 @@ static int catchchildren(void) {
 					p->arguments[0]);
 		} else if( WIFEXITED(status) && WEXITSTATUS(status) != 0 ) {
 			fprintf(stderr,
-"Notification process '%s' returned with exitcode %d!\n",
+"Notification process '%s' returned with exit code %d!\n",
 					p->arguments[0],
 					(int)(WEXITSTATUS(status)));
 		}
@@ -555,10 +559,19 @@ static retvalue startchild(void) {
 			dup2(filedes[0], 0);
 			if( filedes[0] != 0)
 				(void)close(filedes[0]);
+			(void)close(filedes[1]);
 		}
 		/* Try to close all open fd but 0,1,2 */
-		(void)close(filedes[1]);
 		closefrom(3);
+		if( p->causingfile != NULL )
+			setenv("REPREPRO_CAUSING_FILE", p->causingfile, true);
+		else
+			unsetenv("REPREPRO_CAUSING_FILE");
+		setenv("REPREPRO_BASE_DIR", global.basedir, true);
+		setenv("REPREPRO_OUT_DIR", global.outdir, true);
+		setenv("REPREPRO_CONF_DIR", global.confdir, true);
+		setenv("REPREPRO_DIST_DIR", global.distdir, true);
+		setenv("REPREPRO_LOG_DIR", global.logdir, true);
 		(void)execv(p->arguments[0], p->arguments);
 		fprintf(stderr, "Error executing '%s': %s\n", p->arguments[0],
 				strerror(errno));
@@ -674,12 +687,23 @@ static retvalue notificator_enqueuechanges(struct notificator *n,const char *cod
 		p->next = malloc(sizeof(struct notification_process));
 		p = p->next;
 	}
-	if( p == NULL ) {
+	if( FAILEDTOALLOC(p) ) {
 		for( j = 0 ; j < count ; j++ )
 			free(arguments[j]);
 		free(arguments);
 		return RET_ERROR_OOM;
 	}
+	if( causingfile != NULL ) {
+		p->causingfile = strdup(causingfile);
+		if( FAILEDTOALLOC(p->causingfile) ) {
+			for( j = 0 ; j < count ; j++ )
+				free(arguments[j]);
+			free(arguments);
+			free(p);
+			return RET_ERROR_OOM;
+		}
+	} else
+		p->causingfile = NULL;
 	p->arguments = arguments;
 	p->next = NULL;
 	p->child = 0;
@@ -797,13 +821,25 @@ static retvalue notificator_enqueue(struct notificator *n, struct target *target
 		p->next = malloc(sizeof(struct notification_process));
 		p = p->next;
 	}
-	if( p == NULL ) {
+	if( FAILEDTOALLOC(p) ) {
 		size_t j;
 		for( j = 0 ; j < count ; j++ )
 			free(arguments[j]);
 		free(arguments);
 		return RET_ERROR_OOM;
 	}
+	if( causingfile != NULL ) {
+		size_t j;
+		p->causingfile = strdup(causingfile);
+		if( FAILEDTOALLOC(p->causingfile) ) {
+			for( j = 0 ; j < count ; j++ )
+				free(arguments[j]);
+			free(arguments);
+			free(p);
+			return RET_ERROR_OOM;
+		}
+	} else
+		p->causingfile = NULL;
 	p->arguments = arguments;
 	p->next = NULL;
 	p->child = 0;
@@ -885,7 +921,7 @@ void logger_free(struct logger *logger) {
 	free(logger);
 }
 
-retvalue logger_init(const char *confdir,const char *logdir,struct configiterator *iter,struct logger **logger_p) {
+retvalue logger_init(struct configiterator *iter, struct logger **logger_p) {
 	struct logger *n;
 	retvalue r;
 	char *logfilename;
@@ -918,7 +954,7 @@ retvalue logger_init(const char *confdir,const char *logdir,struct configiterato
 	}
 	if( logfilename != NULL ) {
 		assert( *logfilename != '\0');
-		r = logfile_reference(logdir, logfilename, &n->logfile);
+		r = logfile_reference(logfilename, &n->logfile);
 		if( RET_WAS_ERROR(r) ) {
 			free(n);
 			return r;
@@ -939,7 +975,7 @@ retvalue logger_init(const char *confdir,const char *logdir,struct configiterato
 		}
 		n->notificators = newnot;
 		r = notificator_parse(&n->notificators[n->notificator_count++],
-				confdir, iter);
+				 iter);
 		if( RET_WAS_ERROR(r) ) {
 			/* a bit ugly: also free the just failed item here */
 			logger_free(n);
