@@ -10,7 +10,10 @@ struct logger;
 
 enum log_action { LOG_PACKAGE_ADD, LOG_PACKAGE_REPLACE, LOG_PACKAGE_REMOVE};
 
-retvalue logger_init(const char *confdir,const char *logdir,struct configiterator *,/*@out@*/struct logger **);
+/* file causing the current logger_log* run */
+extern /*@null@*/ const char *causingfile;
+
+retvalue logger_init(struct configiterator *, /*@out@*/struct logger **);
 void logger_free(/*@only@*/struct logger *);
 
 retvalue logger_prepare(struct logger *logger);
