@@ -23,20 +23,19 @@
 #include <string.h>
 
 #include "error.h"
-#include "mprintf.h"
+#include "names.h"
 #include "optionsfile.h"
 
-void optionsfile_parse(const char *confdir,const struct option *longopts,
-		       void handle_option(int,const char *)) {
+void optionsfile_parse(const struct option *longopts, void handle_option(int,const char *)) {
 	FILE *f;
 	char *filename;
 	char buffer[1000];
 	int linenr = 0;
 	const struct option *option;
 
-	filename = mprintf("%s/options",confdir);
-	if( filename == NULL ) {
-		(void)fputs("Out of memory!\n",stderr);
+	filename = calc_conffile("options");
+	if( FAILEDTOALLOC(filename) ) {
+		(void)fputs("Out of memory!\n", stderr);
 		exit(EXIT_FAILURE);
 	}
 
