@@ -21,7 +21,7 @@ struct overrideinfo;
 void override_free(/*@only@*//*@null@*/struct overrideinfo *info);
 /* when filename does not start with '/' read override info from overridedir/filename,
  * otherwise from filename directly.. */
-retvalue override_read(const char *confdir, const char *overridedir,const char *filename,/*@out@*/struct overrideinfo **info);
+retvalue override_read(const char *filename, /*@out@*/struct overrideinfo **info);
 
 /*@null@*//*@dependent@*/const struct overrideinfo *override_search(/*@null@*/const struct overrideinfo *overrides,const char *package);
 /*@null@*//*@dependent@*/const char *override_get(/*@null@*/const struct overrideinfo *override,const char *field);
