@@ -59,6 +59,9 @@ struct release {
 	bool new;
 	/* snapshot */
 	bool snapshot;
+	/*@null@*/char *fakesuite;
+	/*@null@*/char *fakecodename;
+	/*@null@*/const char *fakecomponentprefix;
 	/* the files yet for the list */
 	struct release_entry {
 		struct release_entry *next;
@@ -78,6 +81,8 @@ void release_free(struct release *release) {
 	struct release_entry *e;
 
 	free(release->dirofdist);
+	free(release->fakesuite);
+	free(release->fakecodename);
 	while( (e = release->files) != NULL ) {
 		release->files = e->next;
 		free(e->relativefilename);
@@ -122,20 +127,54 @@ static retvalue newreleaseentry(struct release *release, /*@only@*/ char *relati
 	return RET_OK;
 }
 
-retvalue release_init(struct release **release, struct database *database, const char *distdir, const char *codename) {
+retvalue release_init(struct release **release, struct database *database, const char *codename, const char *suite, const char *fakecomponentprefix) {
 	struct release *n;
+	size_t len, suitelen, codenamelen;
 	retvalue r;
 
 	n = calloc(1,sizeof(struct release));
-	n->dirofdist = calc_dirconcat(distdir,codename);
+	n->dirofdist = calc_dirconcat(global.distdir, codename);
 	if( n->dirofdist == NULL ) {
 		free(n);
 		return RET_ERROR_OOM;
 	}
+	if( fakecomponentprefix != NULL ) {
+		len = strlen(fakecomponentprefix);
+		codenamelen = strlen(codename);
+
+		n->fakecomponentprefix = fakecomponentprefix;
+		if( codenamelen > len &&
+		    codename[codenamelen - len - 1] == '/' &&
+		    memcmp(codename + (codenamelen - len),
+		           fakecomponentprefix, len) == 0) {
+			n->fakecodename = strndup(codename,
+					codenamelen - len - 1);
+			if( FAILEDTOALLOC(n->fakecodename) ) {
+				free(n->dirofdist);
+				free(n);
+				return RET_ERROR_OOM;
+			}
+		}
+		if( suite != NULL && (suitelen = strlen(suite)) > len &&
+		    suite[suitelen - len - 1] == '/' &&
+		    memcmp(suite + (suitelen - len),
+		           fakecomponentprefix, len) == 0) {
+			n->fakesuite = strndup(suite,
+					suitelen - len - 1);
+			if( FAILEDTOALLOC(n->fakesuite) ) {
+				free(n->fakecodename);
+				free(n->dirofdist);
+				free(n);
+				return RET_ERROR_OOM;
+			}
+		}
+	}
 	r = database_openreleasecache(database, codename, &n->cachedb);
 	assert( r != RET_NOTHING );
 	if( RET_WAS_ERROR(r) ) {
 		n->cachedb = NULL;
+		free(n->fakecodename);
+		free(n->fakesuite);
 		free(n->dirofdist);
 		free(n);
 		return r;
@@ -144,15 +183,25 @@ retvalue release_init(struct release **release, struct database *database, const
 	return RET_OK;
 }
 
-retvalue release_initsnapshot(const char *distdir, const char *codename, const char *name, struct release **release) {
+retvalue release_initsnapshot(const char *codename, const char *name, struct release **release) {
 	struct release *n;
 
 	n = calloc(1,sizeof(struct release));
-	n->dirofdist = mprintf("%s/%s/snapshots/%s",distdir,codename,name);
+	n->dirofdist = calc_snapshotbasedir(codename, name);
 	if( n->dirofdist == NULL ) {
 		free(n);
 		return RET_ERROR_OOM;
 	}
+	/* apt only removes the last /... part but we create two,
+	 * so stop it generating warnings by faking a suite */
+	n->fakesuite = mprintf("%s/snapshots/%s", codename, name);
+	if( n->fakesuite == NULL ) {
+		free(n->dirofdist);
+		free(n);
+		return RET_ERROR_OOM;
+	}
+	n->fakecodename = NULL;
+	n->fakecomponentprefix = NULL;
 	n->cachedb = NULL;
 	n->snapshot = true;
 	*release = n;
@@ -325,7 +374,7 @@ static retvalue release_usecached(struct release *release,
 				r = RET_ERROR_OOM;
 			else
 				r = checksums_complete(&checksums[ic],
-					fullfilename);
+					fullfilename, NULL);
 			if( r == RET_ERROR_WRONG_MD5 ) {
 				fprintf(stderr,
 "WARNING: '%s' is different from recorded checksums.\n"
@@ -436,8 +485,8 @@ static retvalue openfile(const char *dirofdist, struct openfile *f) {
 	f->fd = open(f->fulltemporaryfilename,O_WRONLY|O_CREAT|O_EXCL|O_NOCTTY,0666);
 	if( f->fd < 0 ) {
 		int e = errno;
-		fprintf(stderr,"Error opening file %s for writing: %m\n",
-				f->fulltemporaryfilename);
+		fprintf(stderr, "Error %d opening file %s for writing: %s\n",
+				e, f->fulltemporaryfilename, strerror(e));
 		return RET_ERRNO(e);
 	}
 	return RET_OK;
@@ -459,8 +508,9 @@ static retvalue writetofile(struct openfile *file, const unsigned char *data, si
 			int e = errno;
 			if( e == EAGAIN || e == EINTR )
 				continue;
-			fprintf(stderr,"Error writing to %s: %d=%m\n",
-					file->fullfinalfilename,e);
+			fprintf(stderr, "Error %d writing to %s: %s\n",
+					e, file->fullfinalfilename,
+					strerror(e));
 			return RET_ERRNO(e);
 		}
 	}
@@ -1100,7 +1150,7 @@ retvalue release_prepare(struct release *release, struct distribution *distribut
 	enum checksumtype cs;
 	int i;
 	static const char * const release_checksum_headers[cs_hashCOUNT] =
-		{ "MD5Sum:\n", "SHA1:\n" };
+		{ "MD5Sum:\n", "SHA1:\n", "SHA256:\n" };
 
 	// TODO: check for existance of Release file here first?
 	if( onlyifneeded && !release->new ) {
@@ -1141,13 +1191,20 @@ retvalue release_prepare(struct release *release, struct distribution *distribut
 		writestring(distribution->label);
 		writechar('\n');
 	}
-	if( distribution->suite != NULL ) {
+	if( release->fakesuite != NULL ) {
+		writestring("Suite: ");
+		writestring(release->fakesuite);
+		writechar('\n');
+	} else if( distribution->suite != NULL ) {
 		writestring("Suite: ");
 		writestring(distribution->suite);
 		writechar('\n');
 	}
 	writestring("Codename: ");
-	writestring(distribution->codename);
+	if( release->fakecodename != NULL )
+		writestring(release->fakecodename);
+	else
+		writestring(distribution->codename);
 	if( distribution->version != NULL ) {
 		writestring("\nVersion: ");
 		writestring(distribution->version);
@@ -1165,6 +1222,10 @@ retvalue release_prepare(struct release *release, struct distribution *distribut
 	writestring("\nComponents:");
 	for( i = 0 ; i < distribution->components.count ; i++ ) {
 		writechar(' ');
+		if( release->fakecomponentprefix != NULL ) {
+			writestring(release->fakecomponentprefix);
+			writechar('/');
+		}
 		writestring(distribution->components.values[i]);
 	}
 	if( distribution->description != NULL ) {
@@ -1222,14 +1283,20 @@ retvalue release_finish(/*@only@*/struct release *release, struct distribution *
 			e = unlink(file->fulltemporaryfilename);
 			if( e < 0 ) {
 				e = errno;
-				fprintf(stderr,"Error deleting %s: %m. (Will be ignored)\n",file->fulltemporaryfilename);
+				fprintf(stderr,
+"Error %d deleting %s: %s. (Will be ignored)\n",
+					e, file->fulltemporaryfilename,
+					strerror(e));
 			}
 		} else if( file->fulltemporaryfilename != NULL ) {
 			e = rename(file->fulltemporaryfilename,
 					file->fullfinalfilename);
 			if( e < 0 ) {
 				e = errno;
-				fprintf(stderr,"Error moving %s to %s: %d=%m!\n",file->fulltemporaryfilename,file->fullfinalfilename,e);
+				fprintf(stderr, "Error %d moving %s to %s: %s!\n",
+						e, file->fulltemporaryfilename,
+						file->fullfinalfilename,
+						strerror(e));
 				r = RET_ERRNO(e);
 				/* after something was done, do not stop
 				 * but try to do as much as possible */
