@@ -1,5 +1,5 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2004,2005,2007 Bernhard R. Link
+ *  Copyright (C) 2004,2005,2007,2008 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
@@ -23,6 +23,7 @@
 #include <string.h>
 #include <strings.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <malloc.h>
 #include "error.h"
 #include "ignore.h"
@@ -169,8 +170,8 @@ retvalue target_closepackagesdb(struct target *target) {
 
 /* Remove a package from the given target. If dereferencedfilekeys != NULL, add there the
  * filekeys that lost references */
-retvalue target_removereadpackage(struct target *target, struct logger *logger, struct database *database, const char *name, const char *oldcontrol, const char *oldpversion, struct strlist *dereferencedfilekeys, struct trackingdata *trackingdata) {
-	char *oldpversion_ifunknown = NULL;
+retvalue target_removereadpackage(struct target *target, struct logger *logger, struct database *database, const char *name, const char *oldcontrol, struct strlist *dereferencedfilekeys, struct trackingdata *trackingdata) {
+	char *oldpversion = NULL;
 	struct strlist files;
 	retvalue result,r;
 	char *oldsource,*oldsversion;
@@ -178,20 +179,19 @@ retvalue target_removereadpackage(struct target *target, struct logger *logger,
 	assert( target != NULL && target->packages != NULL );
 	assert( oldcontrol != NULL && name != NULL );
 
-	if( logger != NULL && oldpversion == NULL ) {
+	if( logger != NULL ) {
 		/* need to get the version for logging, if not available */
-		r = target->getversion(target, oldcontrol,
-				&oldpversion_ifunknown);
-		if( RET_IS_OK(r) )
-			oldpversion = oldpversion_ifunknown;
+		r = target->getversion(oldcontrol, &oldpversion);
+		if( !RET_IS_OK(r) )
+			oldpversion = NULL;
 	}
 	r = target->getfilekeys(oldcontrol, &files);
 	if( RET_WAS_ERROR(r) ) {
-		free(oldpversion_ifunknown);
+		free(oldpversion);
 		return r;
 	}
 	if( trackingdata != NULL ) {
-		r = target->getsourceandversion(target, oldcontrol,
+		r = target->getsourceandversion(oldcontrol,
 				name, &oldsource, &oldsversion);
 		if( !RET_IS_OK(r) ) {
 			oldsource = oldsversion = NULL;
@@ -220,13 +220,13 @@ retvalue target_removereadpackage(struct target *target, struct logger *logger,
 		RET_UPDATE(result, r);
 	} else
 		strlist_done(&files);
-	free(oldpversion_ifunknown);
+	free(oldpversion);
 	return result;
 }
 
 /* Remove a package from the given target. If dereferencedfilekeys != NULL, add there the
  * filekeys that lost references */
-retvalue target_removepackage(struct target *target,struct logger *logger,struct database *database,const char *name,const char *oldpversion,struct strlist *dereferencedfilekeys,struct trackingdata *trackingdata) {
+retvalue target_removepackage(struct target *target, struct logger *logger, struct database *database, const char *name, struct strlist *dereferencedfilekeys, struct trackingdata *trackingdata) {
 	char *oldchunk;
 	retvalue r;
 
@@ -243,7 +243,7 @@ retvalue target_removepackage(struct target *target,struct logger *logger,struct
 		return RET_NOTHING;
 	}
 	r = target_removereadpackage(target, logger, database,
-			name, oldchunk, oldpversion, dereferencedfilekeys,
+			name, oldchunk, dereferencedfilekeys,
 			trackingdata);
 	free(oldchunk);
 	return r;
@@ -251,29 +251,31 @@ retvalue target_removepackage(struct target *target,struct logger *logger,struct
 
 
 /* Like target_removepackage, but delete the package record by cursor */
-retvalue target_removepackage_by_cursor(struct target *target, struct logger *logger, struct database *database, struct cursor *cursor, const char *name, const char *control, const char *oldpversion, struct strlist *dereferencedfilekeys, struct trackingdata *trackingdata) {
-	char *oldpversion_ifunknown = NULL;
+retvalue target_removepackage_by_cursor(struct target_cursor *tc, struct logger *logger, struct database *database, struct strlist *dereferencedfilekeys, struct trackingdata *trackingdata) {
+	struct target * const target = tc->target;
+	const char * const name = tc->lastname;
+	const char * const control = tc->lastcontrol;
+	char *oldpversion = NULL;
 	struct strlist files;
 	retvalue result, r;
 	char *oldsource, *oldsversion;
 
-	assert(target != NULL && target->packages != NULL );
-	assert( name != NULL && control != NULL);
+	assert( target != NULL && target->packages != NULL );
+	assert( name != NULL && control != NULL );
 
-	if( logger != NULL && oldpversion == NULL ) {
+	if( logger != NULL ) {
 		/* need to get the version for logging, if not available */
-		r = target->getversion(target, control,
-				&oldpversion_ifunknown);
-		if( RET_IS_OK(r) )
-			oldpversion = oldpversion_ifunknown;
+		r = target->getversion(control, &oldpversion);
+		if( !RET_IS_OK(r) )
+			oldpversion = NULL;
 	}
 	r = target->getfilekeys(control, &files);
 	if( RET_WAS_ERROR(r) ) {
-		free(oldpversion_ifunknown);
+		free(oldpversion);
 		return r;
 	}
 	if( trackingdata != NULL ) {
-		r = target->getsourceandversion(target, control,
+		r = target->getsourceandversion(control,
 				name, &oldsource, &oldsversion);
 		if( !RET_IS_OK(r) ) {
 			oldsource = oldsversion = NULL;
@@ -284,7 +286,7 @@ retvalue target_removepackage_by_cursor(struct target *target, struct logger *lo
 	if( verbose > 0 )
 		printf("removing '%s' from '%s'...\n",
 				name, target->identifier);
-	result = cursor_delete(target->packages, cursor, name, NULL);
+	result = cursor_delete(target->packages, tc->cursor, tc->lastname, NULL);
 	if( RET_IS_OK(result) ) {
 		target->wasmodified = true;
 		if( oldsource != NULL && oldsversion != NULL ) {
@@ -302,7 +304,7 @@ retvalue target_removepackage_by_cursor(struct target *target, struct logger *lo
 		RET_UPDATE(result, r);
 	} else
 		strlist_done(&files);
-	free(oldpversion_ifunknown);
+	free(oldpversion);
 	return result;
 }
 
@@ -387,7 +389,7 @@ retvalue target_addpackage(struct target *target, struct logger *logger, struct
 		oldcontrol = NULL;
 	} else {
 
-		r = target->getversion(target,oldcontrol,&oldpversion);
+		r = target->getversion(oldcontrol, &oldpversion);
 		if( RET_WAS_ERROR(r) && !IGNORING_(brokenold,"Error parsing old version!\n") ) {
 			free(oldcontrol);
 			return r;
@@ -429,7 +431,8 @@ retvalue target_addpackage(struct target *target, struct logger *logger, struct
 				return r;
 			}
 		} else if(trackingdata != NULL) {
-			r = (*target->getsourceandversion)(target,oldcontrol,name,&oldsource,&oldsversion);
+			r = target->getsourceandversion(oldcontrol,
+					name, &oldsource, &oldsversion);
 			if( RET_WAS_ERROR(r) ) {
 				strlist_done(ofk);
 				if( IGNORING_(brokenold,"Error searching for source name of installed version of %s!\n",name)) {
@@ -484,7 +487,7 @@ retvalue target_checkaddpackage(struct target *target, const char *name, const c
 	} else {
 		int versioncmp;
 
-		r = target->getversion(target,oldcontrol,&oldpversion);
+		r = target->getversion(oldcontrol, &oldpversion);
 		if( RET_WAS_ERROR(r) ) {
 			fprintf(stderr,
 "Error extracting version from old '%s' in '%s'. Database corrupted?\n", name, target->identifier);
@@ -545,8 +548,8 @@ retvalue target_checkaddpackage(struct target *target, const char *name, const c
 			return r;
 		}
 		if( tracking ) {
-			r = (*target->getsourceandversion)(target,
-					oldcontrol, name, &oldsource, &oldsversion);
+			r = target->getsourceandversion(oldcontrol,
+					name, &oldsource, &oldsversion);
 			if( RET_WAS_ERROR(r) ) {
 				fprintf(stderr,
 "Error extracting source name and version from '%s' in '%s'. Database corrupted?\n",
@@ -569,11 +572,9 @@ retvalue target_checkaddpackage(struct target *target, const char *name, const c
 
 retvalue target_rereference(struct target *target, struct database *database) {
 	retvalue result,r;
-	struct cursor *cursor;
+	struct target_cursor iterator;
 	const char *package, *control;
 
-	assert(target->packages!=NULL);
-
 	if( verbose > 1 ) {
 		if( verbose > 2 )
 			printf("Unlocking depencies of %s...\n",
@@ -587,11 +588,11 @@ retvalue target_rereference(struct target *target, struct database *database) {
 	if( verbose > 2 )
 		printf("Referencing %s...\n", target->identifier);
 
-	r = table_newglobalcursor(target->packages, &cursor);
+	r = target_openiterator(target, database, READONLY, &iterator);
 	assert( r != RET_NOTHING );
 	if( RET_WAS_ERROR(r) )
 		return r;
-	while( cursor_nexttemp(target->packages, cursor, &package, &control) ) {
+	while( target_nextpackage(&iterator, &package, &control) ) {
 		struct strlist filekeys;
 
 		r = target->getfilekeys(control, &filekeys);
@@ -608,7 +609,7 @@ retvalue target_rereference(struct target *target, struct database *database) {
 		strlist_done(&filekeys);
 		RET_UPDATE(result, r);
 	}
-	r = cursor_close(target->packages, cursor);
+	r = target_closeiterator(&iterator);
 	RET_ENDUPDATE(result, r);
 	return result;
 }
@@ -638,7 +639,7 @@ retvalue package_check(struct database *database, UNUSED(struct distribution *di
 	char *dummy, *version;
 	retvalue result,r;
 
-	r = target->getversion(target, chunk, &version);
+	r = target->getversion(chunk, &version);
 	if( !RET_IS_OK(r) ) {
 		fprintf(stderr, "Error extraction version number from package control info of '%s'!\n", package);
 		if( r == RET_NOTHING )
@@ -646,7 +647,7 @@ retvalue package_check(struct database *database, UNUSED(struct distribution *di
 		return r;
 	}
 	r = target->getinstalldata(target, package, version, chunk, &dummy,
-			&expectedfilekeys, &files);
+			&expectedfilekeys, &files, NULL);
 	if( RET_WAS_ERROR(r) ) {
 		fprintf(stderr, "Error extracting information of package '%s'!\n",
 				package);
@@ -698,11 +699,10 @@ retvalue package_check(struct database *database, UNUSED(struct distribution *di
 
 /* Reapply override information */
 
-retvalue target_reoverride(UNUSED(void *dummy), struct target *target, struct distribution *distribution) {
-	struct cursor *cursor;
+retvalue target_reoverride(struct target *target, struct distribution *distribution, struct database *database) {
+	struct target_cursor iterator;
 	retvalue result, r;
 	const char *package, *controlchunk;
-	struct table *table = target->packages;
 
 	assert(target->packages!=NULL);
 	assert(distribution!=NULL);
@@ -711,11 +711,11 @@ retvalue target_reoverride(UNUSED(void *dummy), struct target *target, struct di
 		fprintf(stderr,"Reapplying overrides packages in '%s'...\n",target->identifier);
 	}
 
-	r = table_newglobalcursor(table, &cursor);
+	r = target_openiterator(target, database, READWRITE, &iterator);
 	if( !RET_IS_OK(r) )
 		return r;
 	result = RET_NOTHING;
-	while( cursor_nexttemp(table, cursor, &package, &controlchunk) ) {
+	while( target_nextpackage(&iterator, &package, &controlchunk) ) {
 		char *newcontrolchunk = NULL;
 
 		r = target->doreoverride(distribution, package, controlchunk,
@@ -727,15 +727,17 @@ retvalue target_reoverride(UNUSED(void *dummy), struct target *target, struct di
 			break;
 		}
 		if( RET_IS_OK(r) ) {
-			r = cursor_replace(table, cursor,
+			r = cursor_replace(target->packages, iterator.cursor,
 				newcontrolchunk, strlen(newcontrolchunk));
 			free(newcontrolchunk);
-			if( RET_WAS_ERROR(r) )
-				return r;
+			if( RET_WAS_ERROR(r) ) {
+				result = r;
+				break;
+			}
 			target->wasmodified = true;
 		}
 	}
-	r = cursor_close(table, cursor);
+	r = target_closeiterator(&iterator);
 	RET_ENDUPDATE(result, r);
 	return result;
 }
@@ -743,7 +745,7 @@ retvalue target_reoverride(UNUSED(void *dummy), struct target *target, struct di
 /* export a database */
 
 retvalue target_export(struct target *target, struct database *database, bool onlyneeded, bool snapshot, struct release *release) {
-	retvalue result,r;
+	retvalue result;
 	bool onlymissing;
 
 	if( verbose > 5 ) {
@@ -753,20 +755,14 @@ retvalue target_export(struct target *target, struct database *database, bool on
 			printf(" exporting '%s'...\n",target->identifier);
 	}
 
-	r = target_initpackagesdb(target, database, READONLY);
-	if( RET_WAS_ERROR(r) )
-		return r;
-
 	/* not exporting if file is already there? */
 	onlymissing = onlyneeded && !target->wasmodified;
 
 	result = export_target(target->relativedirectory,
-				target->packages,
+				target, database,
 				target->exportmode,
 				release,
 				onlymissing, snapshot);
-	r = target_closepackagesdb(target);
-	RET_ENDUPDATE(result,r);
 
 	if( !RET_WAS_ERROR(result) && !snapshot ) {
 		target->saved_wasmodified =
@@ -782,7 +778,7 @@ retvalue package_rerunnotifiers(UNUSED(struct database *da), struct distribution
 	char *version;
 	retvalue r;
 
-	r = target->getversion(target, chunk, &version);
+	r = target->getversion(chunk, &version);
 	if( !RET_IS_OK(r) ) {
 		fprintf(stderr,"Error extraction version number from package control info of '%s'!\n",package);
 		if( r == RET_NOTHING )
