@@ -19,6 +19,7 @@
 #include <assert.h>
 #include <sys/types.h>
 #include <malloc.h>
+#include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
@@ -489,7 +490,7 @@ retvalue tracking_drop(struct database *db, const char *codename, struct strlist
 static retvalue tracking_recreatereferences(trackingdb t, struct database *database) {
 	struct cursor *cursor;
 	retvalue result, r;
-	struct trackedpackage *pkg;
+	struct trackedpackage *pkg IFSTUPIDCC(=NULL);
 	char *id;
 	int i;
 	const char *key, *value, *data;
@@ -616,7 +617,7 @@ static void print(const char *codename,const struct trackedpackage *pkg){
 retvalue tracking_printall(trackingdb t) {
 	struct cursor *cursor;
 	retvalue result, r;
-	struct trackedpackage *pkg;
+	struct trackedpackage *pkg IFSTUPIDCC(=NULL);
 	const char *key, *value, *data;
 	size_t datalen;
 
@@ -674,14 +675,14 @@ retvalue tracking_parse(struct distribution *d, struct configiterator *iter) {
 	if( modecount > 1 ) {
 		fprintf(stderr,
 "Error parsing config file %s, line %u:\n"
-"Only one of 'keep','all' or 'minimal' can be in one Tracking-header.\n",
+"Only one of 'keep','all' or 'minimal' can be in one Tracking header.\n",
 			config_filename(iter), config_line(iter));
 		return RET_ERROR;
 	}
 	if( modecount < 1 ) {
 		fprintf(stderr,
 "Error parsing config file %s, line %u, column %u:\n"
-"Tracking-mode ('keep','all' or 'minimal') expected.\n",
+"Tracking mode ('keep','all' or 'minimal') expected.\n",
 			config_filename(iter), config_line(iter), config_column(iter));
 		return RET_ERROR;
 	}
@@ -980,7 +981,7 @@ retvalue trackingdata_finish(trackingdb tracks, struct trackingdata *d, struct d
 retvalue tracking_tidyall(trackingdb t, struct database *database, struct strlist *dereferenced) {
 	struct cursor *cursor;
 	retvalue result, r;
-	struct trackedpackage *pkg;
+	struct trackedpackage *pkg IFSTUPIDCC(=NULL);
 	const char *key, *value, *data;
 	size_t datalen;
 
@@ -1011,7 +1012,7 @@ retvalue tracking_tidyall(trackingdb t, struct database *database, struct strlis
 retvalue tracking_reset(trackingdb t) {
 	struct cursor *cursor;
 	retvalue result, r;
-	struct trackedpackage *pkg;
+	struct trackedpackage *pkg IFSTUPIDCC(=NULL);
 	const char *key, *value, *data;
 	char *newdata;
 	size_t datalen, newdatalen;
@@ -1050,7 +1051,7 @@ retvalue tracking_reset(trackingdb t) {
 static retvalue tracking_foreachversion(trackingdb t, struct database *db, struct distribution *distribution,  const char *sourcename, retvalue (action)(trackingdb t,struct trackedpackage *,struct database *,struct distribution *,struct strlist *), struct strlist *dereferenced) {
 	struct cursor *cursor;
 	retvalue result, r;
-	struct trackedpackage *pkg;
+	struct trackedpackage *pkg IFSTUPIDCC(=NULL);
 	const char *value, *data;
 	size_t datalen;
 
@@ -1170,7 +1171,7 @@ static retvalue targetremovesourcepackage(trackingdb t, struct trackedpackage *p
 			free(package);
 			continue;
 		}
-		r = target->getsourceandversion(target, control, package,
+		r = target->getsourceandversion(control, package,
 				&source, &version);
 		assert( r != RET_NOTHING );
 		if( RET_WAS_ERROR(r) ) {
@@ -1222,7 +1223,7 @@ static retvalue targetremovesourcepackage(trackingdb t, struct trackedpackage *p
 		/* that is a bit wasteful, as it parses some stuff again, but
 		 * but that is better than reimplementing logger here */
 		r = target_removereadpackage(target, distribution->logger,
-				database, package, control, NULL,
+				database, package, control,
 				dereferenced, NULL);
 		free(control);
 		free(package);
