@@ -1,5 +1,5 @@
 /*  This file is part of "reprepro"
- *  Copyright (C) 2004,2005,2006,2007 Bernhard R. Link
+ *  Copyright (C) 2004,2005,2006,2007,2008 Bernhard R. Link
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
  *  published by the Free Software Foundation.
@@ -16,6 +16,7 @@
 #include <config.h>
 
 #include <malloc.h>
+#include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -61,6 +62,7 @@ struct package_data {
 	 * same validity */
 	struct strlist new_filekeys;
 	struct checksumsarray new_origfiles;
+	enum filetype new_filetype;
 };
 
 struct upgradelist {
@@ -96,7 +98,7 @@ static retvalue save_package_version(struct upgradelist *upgrade, const char *pa
 	retvalue r;
 	struct package_data *package;
 
-	r = (*upgrade->target->getversion)(upgrade->target,chunk,&version);
+	r = upgrade->target->getversion(chunk, &version);
 	if( RET_WAS_ERROR(r) )
 		return r;
 
@@ -129,7 +131,7 @@ static retvalue save_package_version(struct upgradelist *upgrade, const char *pa
 			/* this should only happen if the underlying
 			 * database-method get changed, so just throwing
 			 * out here */
-			fprintf(stderr,"Package-database is not sorted!!!\n");
+			fprintf(stderr, "Package database is not sorted!!!\n");
 			assert(false);
 			exit(EXIT_FAILURE);
 		}
@@ -142,7 +144,7 @@ retvalue upgradelist_initialize(struct upgradelist **ul,struct target *t,struct
 	struct upgradelist *upgrade;
 	retvalue r,r2;
 	const char *packagename, *controlchunk;
-	struct cursor *cursor;
+	struct target_cursor iterator;
 
 	upgrade = calloc(1,sizeof(struct upgradelist));
 	if( upgrade == NULL )
@@ -150,32 +152,20 @@ retvalue upgradelist_initialize(struct upgradelist **ul,struct target *t,struct
 
 	upgrade->target = t;
 
-	r = target_initpackagesdb(t, database, READONLY);
-	if( RET_WAS_ERROR(r) ) {
-		upgradelist_free(upgrade);
-		return r;
-	}
-
 	/* Beginn with the packages currently in the archive */
 
-	r = table_newglobalcursor(t->packages, &cursor);
-	assert( r != RET_NOTHING );
+	r = target_openiterator(t, database, READONLY, &iterator);
 	if( RET_WAS_ERROR(r) ) {
-		r2 = target_closepackagesdb(t);
-		RET_UPDATE(r,r2);
 		upgradelist_free(upgrade);
 		return r;
 	}
-	while( cursor_nexttemp(t->packages, cursor,
-				&packagename, &controlchunk) ) {
+	while( target_nextpackage(&iterator, &packagename, &controlchunk) ) {
 		r2 = save_package_version(upgrade, packagename, controlchunk);
 		RET_UPDATE(r,r2);
 		if( RET_WAS_ERROR(r2) )
 			break;
 	}
-	r2 = cursor_close(t->packages, cursor);
-	RET_UPDATE(r,r2);
-	r2 = target_closepackagesdb(t);
+	r2 = target_closeiterator(&iterator);
 	RET_UPDATE(r,r2);
 
 	if( RET_WAS_ERROR(r) ) {
@@ -213,10 +203,10 @@ static retvalue upgradelist_trypackage(void *data,const char *chunk){
 	upgrade_decision decision;
 	struct package_data *current,*insertafter;
 
-	r = (*upgrade->target->getname)(upgrade->target,chunk,&packagename);
+	r = upgrade->target->getname(chunk, &packagename);
 	if( RET_WAS_ERROR(r) )
 		return r;
-	r = (*upgrade->target->getversion)(upgrade->target,chunk,&version);
+	r = upgrade->target->getversion(chunk, &version);
 	if( RET_WAS_ERROR(r) ) {
 		free(packagename);
 		return r;
@@ -314,7 +304,10 @@ static retvalue upgradelist_trypackage(void *data,const char *chunk){
 		new->new_version = version;
 		new->version = version;
 		version = NULL; //to be sure...
-		r = upgrade->target->getinstalldata(upgrade->target, new->name, new->new_version, chunk, &new->new_control, &new->new_filekeys, &new->new_origfiles);
+		r = upgrade->target->getinstalldata(upgrade->target,
+				new->name, new->new_version, chunk,
+				&new->new_control, &new->new_filekeys,
+				&new->new_origfiles, &new->new_filetype);
 		if( RET_WAS_ERROR(r) ) {
 			package_data_free(new);
 			return RET_ERROR_OOM;
@@ -332,6 +325,7 @@ static retvalue upgradelist_trypackage(void *data,const char *chunk){
 		char *control;
 		struct strlist files;
 		struct checksumsarray origfiles;
+		enum filetype filetype;
 		int versioncmp;
 
 		upgrade->last = current;
@@ -421,7 +415,7 @@ static retvalue upgradelist_trypackage(void *data,const char *chunk){
 //		if( versioncmp >= 0 && current->version == current->version_in_use
 //				&& current->new_version != NULL ) {
 
-		r = upgrade->target->getinstalldata(upgrade->target, packagename, version, chunk, &control, &files, &origfiles);
+		r = upgrade->target->getinstalldata(upgrade->target, packagename, version, chunk, &control, &files, &origfiles, &filetype);
 		free(packagename);
 		if( RET_WAS_ERROR(r) ) {
 			free(version);
@@ -436,6 +430,7 @@ static retvalue upgradelist_trypackage(void *data,const char *chunk){
 		strlist_move(&current->new_filekeys,&files);
 		checksumsarray_move(&current->new_origfiles, &origfiles);
 		free(current->new_control);
+		current->new_filetype = filetype;
 		current->new_control = control;
 	}
 	return RET_OK;
@@ -454,34 +449,24 @@ retvalue upgradelist_update(struct upgradelist *upgrade,struct aptmethod *method
 retvalue upgradelist_pull(struct upgradelist *upgrade,struct target *source,upgrade_decide_function *predecide,void *decide_data,struct database *database) {
 	retvalue result, r;
 	const char *package, *control;
-	struct cursor *cursor;
-
+	struct target_cursor iterator;
 
 	upgrade->last = NULL;
 	upgrade->currentaptmethod = NULL;
 	upgrade->predecide = predecide;
 	upgrade->predecide_data = decide_data;
 
-	r =  target_initpackagesdb(source, database, READONLY);
+	r = target_openiterator(source, database, READONLY, &iterator);
 	if( RET_WAS_ERROR(r) )
 		return r;
-	result = table_newglobalcursor(source->packages, &cursor);
-	assert( result != RET_NOTHING );
-	if( RET_WAS_ERROR(result) ) {
-		r = target_closepackagesdb(source);
-		RET_UPDATE(result,r);
-		return result;
-	}
 	result = RET_NOTHING;
-	while( cursor_nexttemp(source->packages, cursor, &package, &control) ) {
+	while( target_nextpackage(&iterator, &package, &control) ) {
 		r = upgradelist_trypackage(upgrade, control);
 		RET_UPDATE(result, r);
 		if( RET_WAS_ERROR(r) )
 			break;
 	}
-	r = cursor_close(source->packages, cursor);
-	RET_ENDUPDATE(result,r);
-	r = target_closepackagesdb(source);
+	r = target_closeiterator(&iterator);
 	RET_ENDUPDATE(result,r);
 	return result;
 }
@@ -554,7 +539,6 @@ retvalue upgradelist_predelete(struct upgradelist *upgrade,struct logger *logger
 				r = target_removepackage(upgrade->target,
 						logger, database,
 						pkg->name,
-						pkg->version_in_use,
 						dereferencedfilekeys, NULL);
 			RET_UPDATE(result,r);
 			if( RET_WAS_ERROR(r))
@@ -594,7 +578,8 @@ retvalue upgradelist_install(struct upgradelist *upgrade, struct logger *logger,
 						pkg->new_version,
 						pkg->new_control,
 						&pkg->new_filekeys, NULL, true,
-						dereferencedfilekeys, NULL, 0);
+						dereferencedfilekeys, NULL,
+						pkg->new_filetype);
 			}
 			RET_UPDATE(result,r);
 			if( RET_WAS_ERROR(r) )
@@ -607,7 +592,6 @@ retvalue upgradelist_install(struct upgradelist *upgrade, struct logger *logger,
 				r = target_removepackage(upgrade->target,
 						logger, database,
 						pkg->name,
-						pkg->version_in_use,
 						dereferencedfilekeys, NULL);
 			RET_UPDATE(result,r);
 			if( RET_WAS_ERROR(r) )
